	module cpu (/*AUTOARG*/
		// Outputs
		read_data, io_access,instr_addr, io_wr, io_addr,io_dout,
		
		// Inputs
		ram_addr, clk, cs, reset,io_din,instr,ramrd,ack_i,stb_i
	
	);
	`include "stkpc_params.v"

	input 	[4:0] 				ram_addr;	//Ram_address
	input   					clk;		//Clock	
	input						reset;		//Reset
	input  						cs; 		//Chip select
	input	[STKPC_WORD_MSB:0] 	io_din;		//I/O Data in
	input 	[STKPC_WORD_MSB:0] 	instr;		//Instruction from Ram
	input	[STKPC_WORD_MSB:0] 	ramrd;		//Ram read input
	input 						ack_i;		//Wishbone acknowledge signal 
	input						stb_i;		//Wishbone strobe to signal valid cycle


	output 	[STKPC_INST_MSB:0] 	read_data;		//Used for test bench will show the instructions
	output 	[STKPC_PC_MSB:0]   	instr_addr;		//Next instruction address
	output 						io_wr;			// 1= IO Write 0= IO Read
	output						io_access;		// Signal access to IO for WB
	output 	[STKPC_WORD_MSB:0]	io_addr;		//Output address to IO/RAM	
	output 	[STKPC_WORD_MSB:0] 	io_dout;		//Output data to IO/RAM

	  
	 
	// The D and R stacks
	//reg [STKPC_DSTACK_MSB:0]		dstack[0:STKPC_DSTACK_SIZE];	//Data stack
	//reg [STKPC_RSTACK_MSB:0] 		rstack[0:STKPC_RSTACK_SIZE];	//Return stack

	//Signals for monitoring data and return stack
	
	//ALU Regs
	reg	[STKPC_WORD_MSB:0]		alu_a;
	reg	[STKPC_WORD_MSB:0]		alu_b;
	wire[STKPC_WORD_MSB:0]		alu_c;
	
	//Data Stack Regs
	reg	[STKPC_DSTACK_POINTER_MSB:0] 	dsp;    			//Data stack pointer
	reg [STKPC_DSTACK_POINTER_MSB:0] 	_dsp;				//New data stack pointer
	reg [STKPC_DSTACK_MSB:0] 			st0;			 	//Top of stack data (T)
	reg [STKPC_DSTACK_MSB:0] 			_st0;				//New top of stack data (T')
	reg  								dstkW;				//Data stack write enable Reg
	wire 								_dstkW;   			//Data stack write enable
	wire[STKPC_DSTACK_MSB:0] 			st1 ;//= dstack[dsp];	//Next of Stack
	
	//Return Stack Regs
	reg [STKPC_RSTACK_MSB:0] 		rsp;				//Return stack pointer
	reg [STKPC_RSTACK_MSB:0] 		_rsp;				//New return stack pointer
	reg 							rstkW;     			//R stack write enable
	reg [STKPC_RSTACK_MSB:0] 		_rstkD;				//Newest return stack data
	wire[STKPC_RSTACK_MSB:0] 		rst0 ;	//Top of Return stack
	
	//General Regs
	reg [STKPC_PC_MSB:0] 			pc;					//Program counter
	reg [STKPC_PC_MSB:0]			_pc;				//New program counter
	reg [STKPC_PC_MSB:0] 			start_signal;		//Program counter
	reg 							stall;				//Stall signal for IO/RAM Access
	reg [STKPC_PC_MSB:0] 			_instr_addr;		//Next instruction address
	reg								r_io_access;		//IO Access Reg
	reg								halt;				//Halt signal
	reg								halt_r;				//Halt Reg
	reg  							ioWE;				//IO/RAM write enable Reg
	wire 							_io_access;			//IO Access Signal
	wire 							_ioWE;     			//IO/RAM write enable	
	wire [STKPC_PC_MSB:0] 			pc_plus_1;			//PC+1 to determine next instruction
	wire [STKPC_WORD_MSB:0] 		ramrd;				//Ram read input
	
	//Assigns 
	assign 		_ioWE =ioWE;
	assign 		_dstkW=dstkW;
	assign 		instr_addr =_instr_addr;					//Pass address of next instruction
	assign		_io_access=r_io_access;
	assign 		pc_plus_1 = pc + 1;							
	assign 		io_wr = _ioWE;								//IO/Ram write enable
	assign 		io_addr = st0;								//We use top of stack(T) as address
	assign 		io_dout = st1;								//We use next in stack(N) as data 
	assign 		io_access = (_io_access& stall);			//High if instruction is access to IO
	assign 		read_data = instr;							//Every time a instr is taken from RAM, read_data is updated

	//MUX SELECTS
	reg [STKPC_SEL_PC_MSB:0]  		next_pc_mux_sel;
	reg [STKPC_SEL_ALU_A_MSB:0]  	alu_a_mux_sel;
	reg [STKPC_SEL_ALU_B_MSB:0]  	alu_b_mux_sel;
	reg [STKPC_SEL_DSP_MSB:0]  		dsp_mux_sel;
	reg [STKPC_SEL_RSP_MSB:0]  		rsp_mux_sel;
	reg [STKPC_SEL_RSTACK_MSB :0]   rstack_mux_sel;
	reg [STKPC_SEL_ST0_NEW_MSB:0]  	_st0_mux_sel;
	reg [STKPC_SEL_ALUOP_MSB:0]  	alu_op_mux_sel;
	reg [STKPC_FLOW_HALT_MSB:0]  	halt_mux_sel;
	

	//Stall Generator
	always @(*) begin
		if (reset==1)begin
			stall <=0;
		end
		if (ack_i == 1 | _io_access == 0) begin
			stall<=0;
		end
		else if ((stb_i ==1 | _io_access == 1) && ack_i == 0) begin
			stall <= 1;
		end 
	end
	
	// If stall we stay in actual instruction 
	always @(*) begin
		if (stall==1) begin
			_instr_addr <= pc;
		end
		else begin
			_instr_addr <= _pc;
		end
	end
	
	//MUX for PC
	always @(*) begin
		if (start_signal ==0) begin		//We wait for start signal
			_pc <= 0;
		end 
		else begin
			case(next_pc_mux_sel)
				STKPC_SEL_PC_COND_JMP: begin // Conditional Jump
					if (st0==0) begin		//If top of stack =0, then do the jump				
						_pc <= instr[STKPC_ALU_TO_PC_MSB:0];     
					end 
					else begin
						_pc <= pc_plus_1;
					end
				end
				STKPC_SEL_PC_INC: begin		//PC increment 
					_pc <= pc_plus_1;        
				end
				STKPC_SEL_PC_INSTR: begin	//JMP or CALL , PC given by instruction 
					_pc <= instr[STKPC_ALU_TO_PC_MSB:0];        
				end
				STKPC_SEL_PC_RSTACK: begin	//EXIT of a routine, take PC from return stack
					_pc <= rst0[STKPC_RSTACK_PC_MSB:STKPC_RSTACK_PC_LSB];        
				end
				default:
					_pc <= pc;        
			endcase
		end
	end
	  
	//MUX for ALU A
	always @* begin
		if (start_signal ==0) begin				//We wait for start signal
			alu_a <= 0;
		end
		else begin 
			case(alu_a_mux_sel)
				STKPC_SEL_ALU_A_ST0: begin		//Top of data stack
					alu_a <= st0;        	
				end
				STKPC_SEL_ALU_A_ST1: begin		//Next of data stack
					alu_a <= st1;        
				end
				STKPC_SEL_ALU_A_RST0: begin		//Top of return stack 
					alu_a <= rst0;        
				end
				STKPC_SEL_ALU_A_RSP: begin		//Pointer to return stack, for checking stack depth 
					alu_a <= _rsp << 8;        	//Shift right to display rsp in high bits and dsp in low bits
				end
				default:
					alu_a <= alu_a;        
			endcase
		end
	end
	  
	//MUX for ALU B
	always @* begin
		if (start_signal ==0) begin					//We wait for start signal		
			alu_b <= 0;
		end
		else begin 
			case(alu_b_mux_sel)
				STKPC_SEL_ALU_B_ZERO: begin			//Zeros, b'0000 0000 0000 0000'
					alu_b <= STKPC_ZERO_16BITS ;        
				end
				STKPC_SEL_ALU_B_ONES: begin	
					alu_b <= STKPC_ONES_16BITS;   	//Ones, b'1111 1111 1111 1111'     
				end
				STKPC_SEL_ALU_B_ST0: begin			//Next of data stack
					alu_b <= st0;        
				end
				STKPC_SEL_ALU_B_DSP: begin			//Data stack pointer
					alu_b <= _dsp;        
				end
				default:
					alu_b <= alu_b;        
			endcase
		end
	end
	 
	//MUX for DSP
	always @* begin
		if (start_signal ==0) begin			//We wait for start signal
			_dsp <= 0;
		end
		else begin
			case(dsp_mux_sel)
				STKPC_SEL_DSP: begin
					_dsp <= dsp;        
				end
				STKPC_SEL_DSP_INC_1: begin
					_dsp <= dsp + 1;         
				end
				STKPC_SEL_DSP_INC_2: begin
					_dsp <= dsp + 2;        
				end
				STKPC_SEL_DSP_DEC_1: begin
					_dsp <= dsp - 1;         
				end
				default:
					_dsp <= dsp;        
			endcase
		end
	end
	  
	 //MUX for RSP
	always @* begin
		if (start_signal ==0) begin			//We wait for start signal
			_rsp <= 0;
		end 
		else begin
			case(rsp_mux_sel)
				STKPC_SEL_RSP: begin
					_rsp <= rsp;        
				end
				STKPC_SEL_RSP_INC_1: begin
					_rsp <= rsp + 1;         
				end
				STKPC_SEL_RSP_INC_2: begin
					_rsp <= rsp + 2;        
				end
				STKPC_SEL_RSP_DEC_1: begin
					_rsp <= rsp - 1;         
				end
				default:
					_rsp <= rsp;        
			endcase
		end
	end
	  
	 //MUX for Return Stack
	always @* begin
		if (start_signal ==0) begin			//We wait for start signal
			_rstkD <= 0;
		end 
		else begin
			case(rstack_mux_sel)
				STKPC_SEL_RSTACK_PC: begin
					_rstkD <= _pc;	        
				end
				STKPC_SEL_RSTACK_ST0: begin
					_rstkD <= st0;       
				end             
				STKPC_SEL_RSTACK_PC_PLUS_1: begin
					_rstkD <= {pc_plus_1[STKPC_PC_MSB:0], 1'b0};       
				end
				default:
					_rstkD <= _pc;	       
			endcase
		end
	end
		
		//MUX for _ST0
	always @(*) begin
		if (start_signal ==0) begin
			_st0<=0;
		end 
		else begin
			case(_st0_mux_sel)
				STKPC_SEL_ST0_NEW_ALU: begin
					_st0 <= alu_c;	        
				end
				STKPC_SEL_ST0_NEW_RAM_IO: begin
					_st0 <= |st0[15:14] ? io_din : ramrd;	     
				end             
				STKPC_SEL_ST0_NEW_IMM: begin
					_st0 <= { 1'b0, instr[STKPC_IMM_MSB:0] };       
				end
				default:
					_st0 <= st0;	       
			endcase
		end
	end
	  
	  	    // Halt instruction
	always @* begin
		case(halt_mux_sel)
			STKPC_FLOW_HALT_YES: begin
				halt = 1'b1;        
			end
			STKPC_FLOW_HALT_NO: begin
				halt = 1'b0;        
			end
			default: 
			halt = 1'b0;        
		endcase 
	end

	  //Instruction Decoder
	always @* begin

      next_pc_mux_sel=0;
	  alu_a_mux_sel=0;
	  alu_b_mux_sel=0;
	  alu_op_mux_sel=0;
	  dsp_mux_sel=0;
	  rsp_mux_sel=0;
	  rstack_mux_sel=0;
	  _st0_mux_sel=0;
	  halt_mux_sel = 0;
	  rstkW=0;
	  dstkW=0;
	  ioWE=0;
	  r_io_access=0;
	  
	  casez(instr)
	  
			STKPC_OPCODES_LIT: begin
				next_pc_mux_sel=STKPC_SEL_PC_INC;
				alu_a_mux_sel=STKPC_SEL_ALU_A_ST0;
				alu_b_mux_sel=STKPC_SEL_ALU_B_ZERO;
				alu_op_mux_sel=STKPC_SEL_ALUOP_OR;
				dsp_mux_sel=STKPC_SEL_DSP_INC_1;
				rsp_mux_sel=STKPC_SEL_RSP;
				rstack_mux_sel=STKPC_SEL_RSTACK_PC;
				_st0_mux_sel=STKPC_SEL_ST0_NEW_IMM;
				halt_mux_sel = STKPC_FLOW_HALT_NO;
				rstkW=0;
				dstkW=1;
				ioWE=0;
				r_io_access=0;

		  end 
			STKPC_OPCODES_JMP: begin
				next_pc_mux_sel=STKPC_SEL_PC_INSTR;
				alu_a_mux_sel=STKPC_SEL_ALU_A_ST0;
				alu_b_mux_sel=STKPC_SEL_ALU_B_ZERO;
				alu_op_mux_sel=STKPC_SEL_ALUOP_OR;
				dsp_mux_sel=STKPC_SEL_DSP;
				rsp_mux_sel=STKPC_SEL_RSP;
				rstack_mux_sel=STKPC_SEL_RSTACK_PC;
				_st0_mux_sel=STKPC_SEL_ST0_NEW_ALU;
				halt_mux_sel = STKPC_FLOW_HALT_NO;
				rstkW=0;
				dstkW=0;
				ioWE=0;
				r_io_access=0;
		  end
		  /// after conditional jump, we drop st0 and the new st0 is st1
		  //check
			STKPC_OPCODES_COND_JMP: begin
				next_pc_mux_sel=STKPC_SEL_PC_COND_JMP;
				alu_a_mux_sel=STKPC_SEL_ALU_A_ST1;
				alu_b_mux_sel=STKPC_SEL_ALU_B_ZERO;
				alu_op_mux_sel=STKPC_SEL_ALUOP_OR;
				halt_mux_sel = STKPC_FLOW_HALT_NO;
				dsp_mux_sel=STKPC_SEL_DSP_DEC_1;
				rsp_mux_sel=STKPC_SEL_RSP;
				rstack_mux_sel=STKPC_SEL_RSTACK_PC;
				_st0_mux_sel=STKPC_SEL_ST0_NEW_ALU;
				rstkW=0;
				dstkW=0;
				ioWE=0;
				r_io_access=0;
			end
			STKPC_OPCODES_CALL: begin
				next_pc_mux_sel=STKPC_SEL_PC_INSTR;
				alu_a_mux_sel=STKPC_SEL_ALU_A_ST0;
				alu_b_mux_sel=STKPC_SEL_ALU_B_ZERO;
				alu_op_mux_sel=STKPC_SEL_ALUOP_OR;
				halt_mux_sel = STKPC_FLOW_HALT_NO;
				dsp_mux_sel=STKPC_SEL_DSP;
				rsp_mux_sel=STKPC_SEL_RSP_INC_1;
				rstack_mux_sel=STKPC_SEL_RSTACK_PC_PLUS_1;
				_st0_mux_sel=STKPC_SEL_ST0_NEW_ALU;
				rstkW=1;
				dstkW=0;
				ioWE=0;
				r_io_access=0;
			end
			STKPC_OPCODES_NIP: begin
				next_pc_mux_sel=STKPC_SEL_PC_INC;
				alu_a_mux_sel=STKPC_SEL_ALU_A_ST0;
				alu_b_mux_sel=STKPC_SEL_ALU_B_ZERO;
				alu_op_mux_sel=STKPC_SEL_ALUOP_OR;
				dsp_mux_sel=STKPC_SEL_DSP_DEC_1;
				rsp_mux_sel=STKPC_SEL_RSP;
				rstack_mux_sel=STKPC_SEL_RSTACK_PC;
				_st0_mux_sel=STKPC_SEL_ST0_NEW_ALU;
				halt_mux_sel = STKPC_FLOW_HALT_NO;
				rstkW=0;
				dstkW=0;
				ioWE=0;
				r_io_access=0;
			end
			STKPC_OPCODES_ADD: begin
				next_pc_mux_sel=STKPC_SEL_PC_INC;
				alu_a_mux_sel=STKPC_SEL_ALU_A_ST1;
				alu_b_mux_sel=STKPC_SEL_ALU_B_ST0;
				alu_op_mux_sel=STKPC_SEL_ALUOP_ADD;
				dsp_mux_sel=STKPC_SEL_DSP_DEC_1;
				rsp_mux_sel=STKPC_SEL_RSP;
				rstack_mux_sel=STKPC_SEL_RSTACK_PC;
				_st0_mux_sel=STKPC_SEL_ST0_NEW_ALU;
				halt_mux_sel = STKPC_FLOW_HALT_NO;
				rstkW=0;
				dstkW=0;
				ioWE=0;
				r_io_access=0;
			end
			STKPC_OPCODES_AND: begin
				next_pc_mux_sel=STKPC_SEL_PC_INC;
				alu_a_mux_sel=STKPC_SEL_ALU_A_ST1;
				alu_b_mux_sel=STKPC_SEL_ALU_B_ST0;
				alu_op_mux_sel=STKPC_SEL_ALUOP_AND;
				dsp_mux_sel=STKPC_SEL_DSP_DEC_1;
				rsp_mux_sel=STKPC_SEL_RSP;
				rstack_mux_sel=STKPC_SEL_RSTACK_PC;
				_st0_mux_sel=STKPC_SEL_ST0_NEW_ALU;
				halt_mux_sel = STKPC_FLOW_HALT_NO;
				rstkW=0;
				dstkW=0;
				ioWE=0;
				r_io_access=0;
			end
			STKPC_OPCODES_OR: begin
				next_pc_mux_sel=STKPC_SEL_PC_INC;
				alu_a_mux_sel=STKPC_SEL_ALU_A_ST1;
				alu_b_mux_sel=STKPC_SEL_ALU_B_ST0;
				alu_op_mux_sel=STKPC_SEL_ALUOP_OR;
				dsp_mux_sel=STKPC_SEL_DSP_DEC_1;
				rsp_mux_sel=STKPC_SEL_RSP;
				rstack_mux_sel=STKPC_SEL_RSTACK_PC;
				_st0_mux_sel=STKPC_SEL_ST0_NEW_ALU;
				halt_mux_sel = STKPC_FLOW_HALT_NO;
				rstkW=0;
				dstkW=0;
				ioWE=0;
				r_io_access=0;
			end
			STKPC_OPCODES_XOR: begin
				next_pc_mux_sel=STKPC_SEL_PC_INC;
				alu_a_mux_sel=STKPC_SEL_ALU_A_ST1;
				alu_b_mux_sel=STKPC_SEL_ALU_B_ST0;
				alu_op_mux_sel=STKPC_SEL_ALUOP_XOR;
				dsp_mux_sel=STKPC_SEL_DSP_DEC_1;
				rsp_mux_sel=STKPC_SEL_RSP;
				rstack_mux_sel=STKPC_SEL_RSTACK_PC;
				_st0_mux_sel=STKPC_SEL_ST0_NEW_ALU;
				halt_mux_sel = STKPC_FLOW_HALT_NO;
				rstkW=0;
				dstkW=0;
				ioWE=0;
				r_io_access=0;
			end
			STKPC_OPCODES_INVERT: begin
				next_pc_mux_sel=STKPC_SEL_PC_INC;
				alu_a_mux_sel=STKPC_SEL_ALU_A_ST0;
				alu_b_mux_sel=STKPC_SEL_ALU_B_ONES;
				alu_op_mux_sel=STKPC_SEL_ALUOP_XOR;
				dsp_mux_sel=STKPC_SEL_DSP;
				rsp_mux_sel=STKPC_SEL_RSP;
				rstack_mux_sel=STKPC_SEL_RSTACK_PC;
				_st0_mux_sel=STKPC_SEL_ST0_NEW_ALU;
				halt_mux_sel = STKPC_FLOW_HALT_NO;
				rstkW=0;
				dstkW=0;
				ioWE=0;
				r_io_access=0;
			end
			STKPC_OPCODES_OVER: begin
				next_pc_mux_sel=STKPC_SEL_PC_INC;
				alu_a_mux_sel=STKPC_SEL_ALU_A_ST1;
				alu_b_mux_sel=STKPC_SEL_ALU_B_ZERO;
				alu_op_mux_sel=STKPC_SEL_ALUOP_OR;
				dsp_mux_sel=STKPC_SEL_DSP_INC_1;
				rsp_mux_sel=STKPC_SEL_RSP;
				rstack_mux_sel=STKPC_SEL_RSTACK_PC;
				_st0_mux_sel=STKPC_SEL_ST0_NEW_ALU;
				halt_mux_sel = STKPC_FLOW_HALT_NO;
				rstkW=0;
				dstkW=1;
				ioWE=0;
				r_io_access=0;
			end
			STKPC_OPCODES_DUP: begin
				next_pc_mux_sel=STKPC_SEL_PC_INC;
				alu_a_mux_sel=STKPC_SEL_ALU_A_ST0;
				alu_b_mux_sel=STKPC_SEL_ALU_B_ZERO;
				alu_op_mux_sel=STKPC_SEL_ALUOP_OR;
				dsp_mux_sel=STKPC_SEL_DSP_INC_1;
				rsp_mux_sel=STKPC_SEL_RSP;
				rstack_mux_sel=STKPC_SEL_RSTACK_PC;
				_st0_mux_sel=STKPC_SEL_ST0_NEW_ALU;
				halt_mux_sel = STKPC_FLOW_HALT_NO;
				rstkW=0;
				dstkW=1;
				ioWE=0;
				r_io_access=0;
			end
			STKPC_OPCODES_SWAP: begin
				next_pc_mux_sel=STKPC_SEL_PC_INC;
				alu_a_mux_sel=STKPC_SEL_ALU_A_ST1;
				alu_b_mux_sel=STKPC_SEL_ALU_B_ZERO;
				alu_op_mux_sel=STKPC_SEL_ALUOP_OR;
				dsp_mux_sel=STKPC_SEL_DSP;
				rsp_mux_sel=STKPC_SEL_RSP;
				rstack_mux_sel=STKPC_SEL_RSTACK_PC;
				_st0_mux_sel=STKPC_SEL_ST0_NEW_ALU;
				halt_mux_sel = STKPC_FLOW_HALT_NO;
				rstkW=0;
				dstkW=1;
				ioWE=0;
				r_io_access=0;
			end
			STKPC_OPCODES_DROP: begin
				next_pc_mux_sel=STKPC_SEL_PC_INC;
				alu_a_mux_sel=STKPC_SEL_ALU_A_ST1;
				alu_b_mux_sel=STKPC_SEL_ALU_B_ZERO;
				alu_op_mux_sel=STKPC_SEL_ALUOP_OR;
				dsp_mux_sel=STKPC_SEL_DSP_DEC_1;
				rsp_mux_sel=STKPC_SEL_RSP;
				rstack_mux_sel=STKPC_SEL_RSTACK_PC;
				_st0_mux_sel=STKPC_SEL_ST0_NEW_ALU;
				halt_mux_sel = STKPC_FLOW_HALT_NO;
				rstkW=0;
				dstkW=0;
				ioWE=0;
				r_io_access=0;
			end
			STKPC_OPCODES_N_EQ_T : begin
				next_pc_mux_sel=STKPC_SEL_PC_INC;
				alu_a_mux_sel=STKPC_SEL_ALU_A_ST1;
				alu_b_mux_sel=STKPC_SEL_ALU_B_ST0;
				alu_op_mux_sel=STKPC_SEL_ALUOP_EQUAL;
				dsp_mux_sel=STKPC_SEL_DSP_DEC_1;
				rsp_mux_sel=STKPC_SEL_RSP;
				rstack_mux_sel=STKPC_SEL_RSTACK_PC;
				_st0_mux_sel=STKPC_SEL_ST0_NEW_ALU;
				halt_mux_sel = STKPC_FLOW_HALT_NO;
				rstkW=0;
				dstkW=0;
				ioWE=0;
				r_io_access=0;
			end
			STKPC_OPCODES_N_LT_T: begin
				next_pc_mux_sel=STKPC_SEL_PC_INC;
				alu_a_mux_sel=STKPC_SEL_ALU_A_ST1;
				alu_b_mux_sel=STKPC_SEL_ALU_B_ST0;
				alu_op_mux_sel=STKPC_SEL_ALUOP_SLT;
				dsp_mux_sel=STKPC_SEL_DSP_DEC_1;
				rsp_mux_sel=STKPC_SEL_RSP;
				rstack_mux_sel=STKPC_SEL_RSTACK_PC;
				_st0_mux_sel=STKPC_SEL_ST0_NEW_ALU;
				halt_mux_sel = STKPC_FLOW_HALT_NO;
				rstkW=0;
				dstkW=0;
				ioWE=0;
				r_io_access=0;
			end
			STKPC_OPCODES_N_ULT_T: begin
				next_pc_mux_sel=STKPC_SEL_PC_INC;
				alu_a_mux_sel=STKPC_SEL_ALU_A_ST1;
				alu_b_mux_sel=STKPC_SEL_ALU_B_ST0;
				alu_op_mux_sel=STKPC_SEL_ALUOP_SLTU;
				dsp_mux_sel=STKPC_SEL_DSP_DEC_1;
				rsp_mux_sel=STKPC_SEL_RSP;
				rstack_mux_sel=STKPC_SEL_RSTACK_PC;
				_st0_mux_sel=STKPC_SEL_ST0_NEW_ALU;
				halt_mux_sel = STKPC_FLOW_HALT_NO;
				rstkW=0;
				dstkW=0;
				ioWE=0;
				r_io_access=0;
			end
			STKPC_OPCODES_N_SR_T: begin
				next_pc_mux_sel=STKPC_SEL_PC_INC;
				alu_a_mux_sel=STKPC_SEL_ALU_A_ST1;
				alu_b_mux_sel=STKPC_SEL_ALU_B_ST0;
				alu_op_mux_sel=STKPC_SEL_ALUOP_SRL;
				dsp_mux_sel=STKPC_SEL_DSP_DEC_1;
				rsp_mux_sel=STKPC_SEL_RSP;
				rstack_mux_sel=STKPC_SEL_RSTACK_PC;
				_st0_mux_sel=STKPC_SEL_ST0_NEW_ALU;
				halt_mux_sel = STKPC_FLOW_HALT_NO;
				rstkW=0;
				dstkW=0;
				r_io_access=0;
				
			end
			STKPC_OPCODES_N_SL_T: begin
				next_pc_mux_sel=STKPC_SEL_PC_INC;
				alu_a_mux_sel=STKPC_SEL_ALU_A_ST1;
				alu_b_mux_sel=STKPC_SEL_ALU_B_ST0;
				alu_op_mux_sel=STKPC_SEL_ALUOP_SLL;
				dsp_mux_sel=STKPC_SEL_DSP_DEC_1;
				rsp_mux_sel=STKPC_SEL_RSP;
				rstack_mux_sel=STKPC_SEL_RSTACK_PC;
				_st0_mux_sel=STKPC_SEL_ST0_NEW_ALU;
				halt_mux_sel = STKPC_FLOW_HALT_NO;
				rstkW=0;
				dstkW=0;
				ioWE=0;
				r_io_access=0;
			end
			STKPC_OPCODES_SUB1: begin
				next_pc_mux_sel=STKPC_SEL_PC_INC;
				alu_a_mux_sel=STKPC_SEL_ALU_A_ST0;
				alu_b_mux_sel=STKPC_SEL_ALU_B_ONES;
				alu_op_mux_sel=STKPC_SEL_ALUOP_SUB_1;
				dsp_mux_sel=STKPC_SEL_DSP;
				rsp_mux_sel=STKPC_SEL_RSP;
				rstack_mux_sel=STKPC_SEL_RSTACK_PC;
				_st0_mux_sel=STKPC_SEL_ST0_NEW_ALU;
				halt_mux_sel = STKPC_FLOW_HALT_NO;
				rstkW=0;
				dstkW=0;
				ioWE=0;
				r_io_access=0;
			end
			STKPC_OPCODES_STK_DEP: begin
				next_pc_mux_sel=STKPC_SEL_PC_INC;
				alu_a_mux_sel=STKPC_SEL_ALU_A_RSP;
				alu_b_mux_sel=STKPC_SEL_ALU_B_DSP;
				alu_op_mux_sel=STKPC_SEL_ALUOP_ADD;
				dsp_mux_sel=STKPC_SEL_DSP;
				rsp_mux_sel=STKPC_SEL_RSP;
				rstack_mux_sel=STKPC_SEL_RSTACK_PC;
				_st0_mux_sel=STKPC_SEL_ST0_NEW_ALU;
				halt_mux_sel = STKPC_FLOW_HALT_NO;
				rstkW=0;
				dstkW=0;
				ioWE=0;
				r_io_access=0;
			end
			STKPC_OPCODES_RS_CP: begin
				next_pc_mux_sel=STKPC_SEL_PC_INC;
				alu_a_mux_sel=STKPC_SEL_ALU_A_RST0;
				alu_b_mux_sel=STKPC_SEL_ALU_B_ZERO;
				alu_op_mux_sel=STKPC_SEL_ALUOP_OR;
				dsp_mux_sel=STKPC_SEL_DSP_INC_1;
				rsp_mux_sel=STKPC_SEL_RSP;
				rstack_mux_sel=STKPC_SEL_RSTACK_PC;
				_st0_mux_sel=STKPC_SEL_ST0_NEW_ALU;
				halt_mux_sel = STKPC_FLOW_HALT_NO;
				rstkW=0;
				dstkW=1;
				ioWE=0;
				r_io_access=0;
			end
			STKPC_OPCODES_EXIT: begin
				next_pc_mux_sel=STKPC_SEL_PC_RSTACK;
				alu_a_mux_sel=STKPC_SEL_ALU_A_ST0;
				alu_b_mux_sel=STKPC_SEL_ALU_B_ZERO;
				alu_op_mux_sel=STKPC_SEL_ALUOP_OR;
				dsp_mux_sel=STKPC_SEL_DSP;
				rsp_mux_sel=STKPC_SEL_RSP_DEC_1;
				rstack_mux_sel=STKPC_SEL_RSTACK_PC;
				_st0_mux_sel=STKPC_SEL_ST0_NEW_ALU;
				halt_mux_sel = STKPC_FLOW_HALT_NO;
				rstkW=0;
				dstkW=0;
				ioWE=0;
				r_io_access=0;
			end
			STKPC_OPCODES_RS_PUSH: begin
				next_pc_mux_sel=STKPC_SEL_PC_INC;
				alu_a_mux_sel=STKPC_SEL_ALU_A_ST1;
				alu_b_mux_sel=STKPC_SEL_ALU_B_ZERO;
				alu_op_mux_sel=STKPC_SEL_ALUOP_OR;
				dsp_mux_sel=STKPC_SEL_DSP_DEC_1;
				rsp_mux_sel=STKPC_SEL_RSP_INC_1;
				rstack_mux_sel=STKPC_SEL_RSTACK_ST0;
				_st0_mux_sel=STKPC_SEL_ST0_NEW_ALU;
				halt_mux_sel = STKPC_FLOW_HALT_NO;
				rstkW=1;
				dstkW=0;
				ioWE=0;
				r_io_access=0;
			end
			STKPC_OPCODES_RS_POP: begin
				next_pc_mux_sel=STKPC_SEL_PC_INC;
				alu_a_mux_sel=STKPC_SEL_ALU_A_RST0;
				alu_b_mux_sel=STKPC_SEL_ALU_B_ZERO;
				alu_op_mux_sel=STKPC_SEL_ALUOP_OR;
				dsp_mux_sel=STKPC_SEL_DSP_INC_1;
				rsp_mux_sel=STKPC_SEL_RSP_DEC_1;
				rstack_mux_sel=STKPC_SEL_RSTACK_ST0;
				_st0_mux_sel=STKPC_SEL_ST0_NEW_ALU;
				halt_mux_sel = STKPC_FLOW_HALT_NO;
				rstkW=0;
				dstkW=1;
				ioWE=0;
				r_io_access=0;
			end
			STKPC_OPCODES_NOP: begin
				next_pc_mux_sel=STKPC_SEL_PC_INC;
				alu_a_mux_sel=STKPC_SEL_ALU_A_ST0;
				alu_b_mux_sel=STKPC_SEL_ALU_B_ZERO;
				alu_op_mux_sel=STKPC_SEL_ALUOP_OR;
				dsp_mux_sel=STKPC_SEL_DSP;
				rsp_mux_sel=STKPC_SEL_RSP;
				rstack_mux_sel=STKPC_SEL_RSTACK_PC;
				_st0_mux_sel=STKPC_SEL_ST0_NEW_ALU;
				halt_mux_sel = STKPC_FLOW_HALT_NO;
				rstkW=0;
				dstkW=0;
				ioWE=0;
				r_io_access=0;
			end
			STKPC_OPCODES_HALT: begin
				next_pc_mux_sel=STKPC_SEL_PC_INC;
				alu_a_mux_sel=STKPC_SEL_ALU_A_ST0;
				alu_b_mux_sel=STKPC_SEL_ALU_B_ZERO;
				alu_op_mux_sel=STKPC_SEL_ALUOP_OR;
				dsp_mux_sel=STKPC_SEL_DSP;
				rsp_mux_sel=STKPC_SEL_RSP;
				rstack_mux_sel=STKPC_SEL_RSTACK_PC;
				_st0_mux_sel=STKPC_SEL_ST0_NEW_ALU;
				halt_mux_sel = STKPC_FLOW_HALT_YES;
				rstkW=0;
				dstkW=0;
				ioWE=0;
				r_io_access=0;
			end
			STKPC_OPCODES_MEM_WR: begin
				next_pc_mux_sel=STKPC_SEL_PC_INC;
				alu_a_mux_sel=STKPC_SEL_ALU_A_ST0;
				alu_b_mux_sel=STKPC_SEL_ALU_B_ZERO;
				alu_op_mux_sel=STKPC_SEL_ALUOP_OR;
				dsp_mux_sel=STKPC_SEL_DSP;
				rsp_mux_sel=STKPC_SEL_RSP;
				rstack_mux_sel=STKPC_SEL_RSTACK_PC;
				_st0_mux_sel=STKPC_SEL_ST0_NEW_ALU;
				halt_mux_sel = STKPC_FLOW_HALT_NO;
				rstkW=0;
				dstkW=0;
				ioWE=1;
				r_io_access=1;
			end
			STKPC_OPCODES_MEM_RD: begin
				next_pc_mux_sel=STKPC_SEL_PC_INC;
				alu_a_mux_sel=STKPC_SEL_ALU_A_ST0;
				alu_b_mux_sel=STKPC_SEL_ALU_B_ZERO;
				alu_op_mux_sel=STKPC_SEL_ALUOP_OR;
				dsp_mux_sel=STKPC_SEL_DSP_INC_1;
				rsp_mux_sel=STKPC_SEL_RSP;
				rstack_mux_sel=STKPC_SEL_RSTACK_PC;
				_st0_mux_sel=STKPC_SEL_ST0_NEW_RAM_IO;
				halt_mux_sel = STKPC_FLOW_HALT_NO;
				rstkW=0;
				dstkW=1;
				ioWE=0;
				r_io_access=1;
			end
		endcase
	end

	 //update of data and return stack, pushing the top of stack 
	//always @(posedge clk or posedge reset) begin
	//  if (reset!=1'b1) begin
	//    if (_dstkW) begin
	//      dstack[_dsp] = st0;
	//    end                 // 
	//    if (rstkW) begin
	//      rstack[_rsp] = _rstkD;
	//    end                 // 
	//  end                   // 
	//end                     // 
	//

/*//update of data and return stack, pushing the top of stack 	
  always @(posedge clk /* or posedge reset) begin
    // if (reset!=1'b1) begin
     // if (_dstkW) begin
     //   dstack[_dsp] = st0;
    //  end  
      if (rstkW) begin
        rstack[_rsp] = _rstkD;
      end  
    // end 
  end      */
  
  	//Data Stack
	data_stack DATA_STACK (
	   // Outputs
	   .data_out            (st1),
	   // Inputs
	   .data_in             (st0),
	   .addr_rd             (dsp),
	   .addr_wr             (_dsp),
	   .wr_en               (_dstkW),
	   .clk             	(clk)
	);
	
	//Return Stack
	return_stack RETURN_STACK (
	   // Outputs
	   .data_out            (rst0),
	   // Inputs
	   .data_in             (_rstkD),
	   .addr_rd             (rsp),
	   .addr_wr             (_rsp),
	   .wr_en               (rstkW),
	   .clk             	(clk)
	);
	
  
	//ALU
	stkpc_alu STKPC_ALU (
	   // Outputs
	   .c               (alu_c[STKPC_WORD_MSB:0]),
	   // Inputs
	   .a               (alu_a[STKPC_WORD_MSB:0]),
	   .b               (alu_b[STKPC_WORD_MSB:0]),
	   .op              (alu_op_mux_sel[STKPC_SEL_ALUOP_MSB:0])
	);
	
	
		//Update PC, pointers
	always @(posedge clk or posedge reset) begin
		if (reset==1'b1) begin
			pc <= 0;
			dsp <= 0;
			st0 <= 0;
			rsp <= 0;
			halt_r <=0;
		end else if(stall==0) begin	  
			dsp <= _dsp;
			pc <= _pc;
			st0 <= _st0;
			rsp <= _rsp;
			halt_r <= halt;

		end
	end
	//start signal to initiate after initial reset	  
	always @(posedge clk or posedge reset) begin
		if (reset==1'b1) begin
			start_signal<=0;
		end else begin 
			start_signal <=1;
		end
	end

endmodule //cpu
	
