module cpu_top(/*AUTOARG*/
	       // Outputs
	       adr_o, dat_o, we_o, sel_o, stb_o, cyc_o,		
  
	       // Inputs
	       rst_i,clk_i,dat_i,ack_i,err_i,rty_i,cs
	       );
`include "stkpc_params.v"
  
  //Outputs
  output[WB_ADR_O_MSB:0] 	adr_o;
  output [WB_DAT_O_MSB:0]       dat_o;
  output                        we_o;
  output [WB_SEL_O_MSB:0]       sel_o;
  output                        stb_o;
  output                        cyc_o;

  
  //Inputs
  input                         rst_i;
  input                         clk_i;
  input [WB_DAT_I_MSB:0]        dat_i;	
  input                         ack_i;
  input                         err_i;
  input                         rty_i;		
  input                         cs;		

  
  wire [STKPC_WORD_MSB:0]       io_din;		//I/O Data in
  wire [STKPC_WORD_MSB:0]       instr;
  wire [STKPC_WORD_MSB:0]       ramrd;	
  wire                          ack_i;
  wire                          ram_cs;
  wire                          wb_cs;

  wire [STKPC_INST_MSB:0]       read_data;	//Used for test bench will show the instructions
  wire [STKPC_PC_MSB:0]         instr_addr;	//Used for test bench will show the instructions
  wire                          io_rd;				//Read enable
  wire                          io_wr;				//Write enable
  wire [STKPC_WORD_MSB:0]       io_addr;	//Output address to RAM	
  wire [STKPC_WORD_MSB:0]       io_dout;		//Output data to RAM
  wire                          io_access;
  reg                           clk_stall;
  
  always @(*) begin
    if (io_access==1) begin
      clk_stall = io_access^clk_i;
    end else begin
      clk_stall = clk_i;
    end
  end

`ifdef ORIGINAL_RAM
  
  // RAM declaration
  ram2ports #(
	      .WORD_WIDTH(16),
	      .ADDR_WIDTH(5) 							// 32 words
	      )
  U_RAM (
    
	 // Outputs
	 .data_out_a     	(instr[15:0]),  	// Instructions read from ram
	 .data_out_b     	(ramrd[15:0]),  	// Instructions read from ram
    
	 // Inputs
	 .clock            	(clk_i),                	// Clock
	 .addr_a          	(instr_addr[4:0]),              	//PC used as address      
	 .data_in_a        	(16'd0),               // We won't write to RAM
	 .cs_a            	(cs),					//Always selected
	 .we_a             	(1'b0),

	 .addr_b           	(io_addr [5:1]),              	//PC used as address      
	 .data_in_b       	(io_dout [15:0]),               // We won't write to RAM
	 .cs_b             	(ram_cs),					//Always selected
	 .we_b             	(io_wr)
	 );               //We won't write to RAM
`else // !`ifdef ORIGINAL_RAM
  `ifdef RAM_130NM
  // 11-bit address on target RAM (32-bit wide)
  
  mem_wrapper U_CODE_RAM (
                          // Outputs
                          .read_data           (instr[15:0]),
                          // Inputs
                          .clock               (clk_i),                   // Templated
                          .addr                ({11'b0,instr_addr [4:0]}), // Templated
                          .write_data          (16'b0), // Templated
                          .cs                  (1'b1),                  // Templated
                          .we                  (1'b0)); //

  mem_wrapper U_DATA_RAM (
                          // Outputs
                          .read_data           (ramrd[15:0]),
                          // Inputs
                          .clock               (clk_i),                   // Templated
                          .addr                (io_addr [15:0]), // Templated
                          .write_data          (io_dout [15:0]), // Templated
                          .cs                  (ram_cs),                  // Templated
                          .we                  (io_wr)); // 


  
  `endif //  `ifdef RAM_130NM
  
 `endif // !`ifdef ORIGINAL_RAM
  


  
  cpu CPU (

	   // Outputs
	   .read_data   		(read_data[15:0]),  	// Instructions read from ram
	   .instr_addr  		(instr_addr[STKPC_PC_MSB:0]),
	   .io_wr       		(io_wr),  	// Instructions read from ram
	   .io_addr     		(io_addr[15:0]),  	// Instructions read from ram
	   .io_dout     		(io_dout[15:0]),  	// Instructions read from ram
	   .io_access    		(io_access),  	// Instructions read from ram

	   // Inputs
	   .clk        			(clk_i),                	// Clock     
	   .reset     			(rst_i),               // We won't write to RAM
	   .cs             		(cs),					//Always selected
	   .io_din            	(io_din),
	   .instr           	(instr[15:0]),
	   .ack_i				(ack_i),
	   .stb_i				(stb_o),		 //PC used as address      
	   .ramrd				(ramrd[15:0])
	   );               //We won't write to RAM




  

  
  wb_m_16 WB_M_16 (

		   // Outputs
		   .adr_o   			(adr_o),  	// Instructions read from ram
		   .dat_o  			(dat_o),
		   .we_o       		(we_o),  	// Instructions read from ram
		   .sel_o       		(sel_o),  	// Instructions read from ram
		   .stb_o     		(stb_o),  	// Instructions read from ram
		   .cyc_o     		(cyc_o),  	// Instructions read from ram
		   .dat_i_cpu    		(io_din),  	// Instructions read from ram

		   // Inputs
		   .clk_i        		(clk_i),                	// Clock     
		   .rst_i     		(rst_i),               // We won't write to RAM
		   .dat_i             (dat_i),					//Always selected
		   .ack_i            	(ack_i),
		   .err_i     		(err_i),               // We won't write to RAM
		   .rty_i             (rty_i),					//Always selected
		   .dat_o_cpu         (io_dout),
		   .adr_o_cpu         (io_addr),              	//PC used as address      
		   .we_i_cpu     		(io_wr),               // We won't write to RAM
		   .io_access    		(io_access),               // We won't write to RAM
		   .cs             	(cs )
	           );   
  
  adr_dec ADR_DEC (

		   // Outputs
		   .wb_cs   			(wb_cs),  	// Instructions read from ram
		   .ram_cs  			(ram_cs ),

		   // Inputs
		   .clk_i        		(clk_i),                	// Clock     
		   .rst_i     		(rst_i),               // We won't write to RAM
		   .io_adr            (io_addr),
		   .io_wr    			(io_wr)
	           );   
  
endmodule //cpu_top

