	// Generic RAM model
	`timescale 1ns/1ps

	module ram2ports (/*AUTOARG*/
		// Outputs
		data_out_a,data_out_b,
		// Inputs
		clock, addr_a, addr_b, data_in_a,data_in_b, cs_a, we_a, cs_b, we_b
	);
	parameter WORD_WIDTH = 16;
	parameter ADDR_WIDTH = 8;
	parameter RAM_SIZE = 1<< ADDR_WIDTH;
	localparam WORD_MSB = WORD_WIDTH-1;
	localparam ADDR_MSB = ADDR_WIDTH-1;

	output [WORD_MSB:0] data_out_a;
	output [WORD_MSB:0] data_out_b;


	input clock;
	input [ADDR_MSB:0] addr_a;
	input [WORD_MSB:0] data_in_a;
	input [ADDR_MSB:0] addr_b;
	input [WORD_MSB:0] data_in_b;
	input              cs_a;
	input              we_a;
	input              cs_b;
	input              we_b;
	
	reg [WORD_MSB:0] data_out_a;
	reg [WORD_MSB:0] data_out_b;
	reg [WORD_MSB:0]     mem0 [0:RAM_SIZE-1];
	reg [ADDR_MSB:0] addr_a_latched;
	reg [ADDR_MSB:0] addr_b_latched;
	
	event dbg_addr_a;
		  

	always @(posedge clock) begin
		if(cs_a) begin
			addr_a_latched <= addr_a;
		end
		if(cs_b) begin
		addr_b_latched <= addr_b;
		end
	end
		
	always @(addr_a_latched) begin
		-> dbg_addr_a;                                // 
		data_out_a <= mem0[addr_a_latched[ADDR_MSB:0]]; // 
	end
		  
	always @(addr_b_latched) begin
		data_out_b <= mem0[addr_b_latched[ADDR_MSB:0]];
	end
				
		  // Write
		
	always @(posedge clock) begin
		if(cs_a && we_a) begin
			$display("Memory write A ");
			mem0[addr_a] <= data_in_a;
		end
		if(cs_b && we_b) begin
			 $display("Memory write B");
			 mem0[addr_b] <= data_in_b;
		end
	end
		
endmodule // ram2ports
		
		
	
