	module stkpc_alu (
		// Outputs
		c,		
		// Inputs
		a,b,op
	);
  
	`include "stkpc_params.v"

	//Outputs
	output[STKPC_WORD_MSB:0] 		c;
	//Inputs
	input [STKPC_WORD_MSB:0] 		a; 
	input [STKPC_WORD_MSB:0]		b;
	input [STKPC_SEL_ALUOP_MSB:0] 	op;

	reg [STKPC_WORD_MSB:0] 		c;
	reg  [STKPC_WORD_MSB:0] 	adder_a; 
	reg  [STKPC_WORD_MSB:0] 	adder_b;
	wire [STKPC_WORD_MSB:0]  	adder_res;
	wire                  		adder_cin;


  
	always @* begin
		adder_a = a;
		adder_b = b;
		c = adder_res;    

		case(op)
			STKPC_SEL_ALUOP_ADD: begin		//Addition	
				c = a+b;    
			end
			STKPC_SEL_ALUOP_SRL: begin		//Shift register right
				c = a >> b[3:0];
			end
			STKPC_SEL_ALUOP_SLL: begin		//Shift register left
				c = a << b[3:0];
			end
			STKPC_SEL_ALUOP_AND: begin		//AND
				c = a & b;
			end
			STKPC_SEL_ALUOP_OR: begin		//OR
				c = a | b;
			end
			STKPC_SEL_ALUOP_XOR: begin		//XOR
				c = a ^ b;
			end
			STKPC_SEL_ALUOP_NOT: begin		//NOT
				c = !a;
			end
			STKPC_SEL_ALUOP_SUB: begin		//Subtraction
				c = a-b;
			end
			STKPC_SEL_ALUOP_SUB_1: begin	//Unitary subtraction
				c = a-(|b);
			end
			STKPC_SEL_ALUOP_EQUAL: begin	//Equal
				c = {16{!(|(a-b))}};//change to equality 
			end
			STKPC_SEL_ALUOP_SLT: begin		//Less than
				c = {16{($signed(a) < $signed(b))}};
			end
			STKPC_SEL_ALUOP_SLTU: begin		//Unsigned less than
				c = {16{ a < b }};
			end

			default:begin
				adder_a = a;
				adder_b = b;
				c = adder_res;    
			end
		endcase // case (op)    
	end
  
	stkpc_generic_adder 
			#(.WIDTH(STKPC_WORD_BITS)) 
	U_ALU_ADDER (             
		// Outputs
		.res              (adder_res[STKPC_WORD_BITS-1:0]),
		.cout             (adder_cout),
		// Inputs
		.a                (adder_a[STKPC_WORD_BITS-1:0]),
		.b                (adder_b[STKPC_WORD_BITS-1:0]),
		.cin              (adder_cin)
	);
  
  
endmodule // stkpc_alu

