	module wb_s_16 (/*AUTOARG*/
		// Outputs
		dat_o, ack_o,err_o,rty_o,reg_f0,reg_f1,reg_f2,reg_f3,reg_f4,reg_f5,

		// Inputs
		rst_i,clk_i,dat_i,adr_i,cyc_i,we_i,stb_i,sel_i,cs
	);
  
`include "stkpc_params.v"

	//Outputs						//WB Signals
	output[WB_DAT_O_MSB:0] 	dat_o;	//Data out
	output					ack_o;	//Acknowledgement out
	output				 	rty_o;	//Ready out
	output					err_o;	//Error out, not used for simple design
	output[WB_DAT_O_MSB:0]	reg_f0;	//Outputs used to monitor the register file
	output[WB_DAT_O_MSB:0]	reg_f1;
	output[WB_DAT_O_MSB:0]	reg_f2;
	output[WB_DAT_O_MSB:0]	reg_f3;
	output[WB_DAT_O_MSB:0]	reg_f4;
	output[WB_DAT_O_MSB:0]	reg_f5;
  
	//Inputs 							//WB Signals
	input						rst_i;	//Reset
	input						clk_i;	//Clock
	input	[WB_DAT_O_MSB:0] 	dat_i;	//Data in
	input	[WB_ADR_O_MSB:0]	adr_i;	//Address in
	input						cyc_i;	//Cycle input, to notifies a cycle is in progress if high
	input						we_i;	//Write enable
	input						stb_i;	//Strobe in, notifies the specific slave is targeted
	input	[WB_SEL_O_MSB:0]	sel_i;	//Select in, used for arranging data written to dat_i, not used in this case we transmit 16 bits to 16 bits
	input						cs;		//Chip select

	reg [WB_DAT_O_MSB:0] 		reg_file [0:GPIO_REG_FILE_MSB];		//Register file to store data
	reg [WB_DAT_O_MSB:0] 		dat_o;
	//reg							ack_o;
	reg							rty_o;
	reg							err_o;
	// reg [WB_DELAY_MSB:0]		wcet; 		//Worst case execution time, max cycles of delay needed for slave operation
	reg	[WB_DELAY_MSB:0]		temp_count;		//Temporal to count cycles
	reg							ready;			//Signal to notify that slave finish its processing and can transmit back
	
	reg 						r_ack_o;
	wire 						_ack_o;
	
	assign reg_f0 = reg_file[0];	//Used to monitor register file in simulation
	assign reg_f1 = reg_file[1];
	assign reg_f2 = reg_file[2];
	assign reg_f3 = reg_file[3];
	assign reg_f4 = reg_file[4];
	assign reg_f5 = reg_file[5];
	assign _ack_o = r_ack_o;
	assign ack_o = _ack_o & stb_i;


  wire  [WB_DELAY_MSB:0]                                  wcet; 		//Worst case execution time, max cycles of delay needed for slave operation

  assign wcet = 'd1;

  
	/*always @* begin
		ack_o <= _ack_o & stb_i;
	end
*/
	//Delay generator 
  always @(posedge clk_i or posedge rst_i) begin
    if(rst_i == 1'b1) begin
      ready <= 0;
      temp_count <= 0;
    end 
    else begin
      if (stb_i==1) begin
	temp_count	<= temp_count+1;
	if (temp_count == wcet) begin
	  ready <= 1;           // 
	  temp_count <= 0;
	end                     // 
      end                       // 
      else begin                // 
	temp_count <= 0;
	ready <= 0; // 
      end           // 
    end             //
  end // always @ (posedge clk_i or posedge rst_i)
  
  
   //Data in and out
  always @(posedge clk_i or posedge rst_i) begin
    if(rst_i==1'b1) begin
      dat_o <= 0;
      r_ack_o <= 0;
      rty_o <= 0;
      err_o <= 0;
    end else if(cs) begin	//If WB is the target
      if(wcet==0) begin 
	if (we_i==0 && stb_i==1) begin 	//Master is going to read 
	  dat_o<=  reg_file[adr_i&255];
	  r_ack_o<= 1;
	end else if (we_i == 1 && stb_i == 1 ) begin	//Master is going to write
	  reg_file[adr_i&255]<=  dat_i;                 // 
	  r_ack_o<=  1;                                 // 
	end else begin //if we are reading and slave is ready
	  r_ack_o<=0;  // 
	end  
      end
      else begin
	if (we_i==0 && stb_i==1&& ready ==1) begin 	//Master is going to read 
	  dat_o<=  reg_file[adr_i&255];                 // 
	  r_ack_o<= 1;                                  // 
	end else if (we_i == 1 && stb_i == 1 && ready == 1) begin	//Master is going to write
	  reg_file[adr_i&255]<=  dat_i; // 
	  r_ack_o<=  1;
	end else begin //if we are reading and slave is ready
	  r_ack_o<=0;  // 
	end 
      end
    end
  end
	  
 endmodule //wb_s_16
