`timescale 1ns/1ps
module tb;
`include "useful_tasks.v"  // some helper tasks
`include "stkpc_params.v"

  reg rst_async; // asynchronous reset
  wire rst_n; // synchronous reset (falling edge)
  
  wire clk;
  reg  [15:0] io_din;

  reg [4:0] ram_addr;
  wire [15:0] read_data;
  reg [1:0]	test;

  
  /* chip  AUTO_TEMPLATE(
   .cs                                  (1'b1), // Continuous access to the RAM
   .reset                               (rst_async), 
   ); */
  chip  U_CHIP 
  (
   .adr_o                               (),
   .dat_o                               (),
   .we_o                                (),
   .sel_o                               (),
   .stb_o                               (),
   .cyc_o                               (),
   /*AUTOINST*/
   // Inputs
   .clk                                 (clk),
   .reset                               (rst_async),             // Templated
   .cs                                  (1'b1));                  // Templated
  
   clock_gen U_CLK_GEN (
                          /*AUTOINST*/
                        // Outputs
                        .clk            (clk));
  
  
   
 
  
 

  task load_program_memory;
    reg [1024:0] filename;
    reg [7:0]   memory [2047:0]; // 16-bitx32 memory
    integer      i;
    integer      dummy;
    begin

      filename = 0;
      dummy = $value$plusargs("program_memory=%s", filename);
      if(filename ==0) begin
        $display("WARNING! No content specified for program memory");
      end
      else begin
        for(i=0; i<2048; i=i+1) begin
          memory[i] = 0;
        end
        $readmemh (filename, memory);

`ifdef ORIGINAL_RAM        
        for(i=0; i<(2048); i=i+1) begin
          U_CHIP.CPU_TOP.U_RAM.mem0[i] = {memory[2*i+1],memory[2*i]};
        end
`else
        for(i=0; i<(2048); i=i+1) begin
          // Warning - Momory is 32-bit wide but we use only 16-bit for now
          U_CHIP.CPU_TOP.U_CODE_RAM.U_RAM1536X32.mem[i] = {8'd0,8'd0,memory[2*i+1],memory[2*i]};
        end
`endif
        
      end
    end
  endtask // load_program_memory

  
    task display_program_memory;
    integer      i;
    begin

`ifdef ORIGINAL_RAM              
        for(i=0; i<(8); i=i+1) begin
           $display("%x\t",U_CHIP.CPU_TOP.U_RAM.mem0[i]);
        end
`else
      for(i=0; i<(8); i=i+1) begin
         $display("%x\t",U_CHIP.CPU_TOP.U_CODE_RAM.U_RAM1536X32.mem[i]);
      end
`endif
      
      end
    
  endtask // load_program_memory


  
  // Dump all nets to a vcd file called tb.vcd
  initial
    begin
      load_program_memory;
	  display_program_memory;
	$dumpfile("tb.vcd");
	$dumpvars(0,tb);
     end

  
event evt_finish;

  always @(posedge U_CHIP.CPU_TOP.CPU.halt_r) begin
    $display("-I- Simulation halted by the CPU - <halt> instr");
    -> evt_finish; 
    #10;    
    $finish;
  end


integer i;
   always @(evt_finish) begin
       $display("new_st0=0x%04h  st0=0x%04h st1=0x%04h  rst0=0x%04h \n pc=0x%04h dsp=0x%04h rsp=0x%04h instruction=0x%04h ",
                U_CHIP.CPU_TOP.CPU._st0,
				U_CHIP.CPU_TOP.CPU.st0,
                U_CHIP.CPU_TOP.CPU.st1,
                U_CHIP.CPU_TOP.CPU.rst0,
				U_CHIP.CPU_TOP.CPU.pc,
                U_CHIP.CPU_TOP.CPU.dsp,
                U_CHIP.CPU_TOP.CPU.rsp,
				U_CHIP.CPU_TOP.CPU.instr
				);
		for(i=0; i<15; i=i+4) begin        
       $display("dstack[%02d]=0x%04h  dstack[%02d]=0x%04h  dstack[%02d]=0x%04h dstack[%02d]=0x%04h",
                i,U_CHIP.CPU_TOP.CPU.DATA_STACK.dstack[i],i+1,U_CHIP.CPU_TOP.CPU.DATA_STACK.dstack[i+1],
				i+2,U_CHIP.CPU_TOP.CPU.DATA_STACK.dstack[i+2],i+3,U_CHIP.CPU_TOP.CPU.DATA_STACK.dstack[i+3]);
       end
	   		for(i=0; i<15; i=i+4) begin        
       $display("rstack[%02d]=0x%04h  rstack[%02d]=0x%04h  rstack[%02d]=0x%04h rstack[%02d]=0x%04h",
                i,U_CHIP.CPU_TOP.CPU.RETURN_STACK.rstack[i],i+1,U_CHIP.CPU_TOP.CPU.RETURN_STACK.rstack[i+1],
				i+2,U_CHIP.CPU_TOP.CPU.RETURN_STACK.rstack[i+2],i+3,U_CHIP.CPU_TOP.CPU.RETURN_STACK.rstack[i+3]);
       end
     end
  
  // Start by pulsing the reset low for some nanoseconds
  initial begin
    rst_async = 1'b1;
    io_din =0;

    
    #200;
    rst_async = 1'b0;


    //@(negedge clk);
         
    @(negedge clk); 	       
    @(negedge clk);		     
	@(negedge clk);	    
	@(negedge clk);	  
	@(negedge clk);	     
	@(negedge clk);		      
	@(negedge clk);	     
	@(negedge clk);	     
	@(negedge clk);	      
	@(negedge clk);	    
	@(negedge clk);	     
	@(negedge clk);	     
	@(negedge clk);	  
	@(negedge clk);
	@(negedge clk);
	@(negedge clk);
	@(negedge clk);	
	@(negedge clk);
	@(negedge clk);
	@(negedge clk);
	@(negedge clk);
	@(negedge clk);
	@(negedge clk);
    test= 2'b11;
	$display("-I- Ram output : %x, %x, %x, %x",test[1],test[0],test,{test[1], test[1], test[1], test});
	
    $display("-I- Done !");

    $finish;
  end


endmodule // tb
/*  
 Local Variables:
 verilog-library-directories:(
 "."
 "../../rtl/module/verilog"
 )
 End:
 */
