%{
/*
 * CTM  machine description file.
 * Greatly inspired by XR16 LCC port
 *
 * Registers    Use
 * r0           zero; unchangeable
 * r1           reserved for the assembler
 * r2           function return value
 * r3-r5        first function arg(s)
 * r6-r9        scratch registers
 * r10-r12      register variables
 * r13          stack pointer
 * r14          interrupt return address (reserved)
 * r15          return address
 * 
 * Operator terminals generated by ops c=1 s=2 i=2 l=4 h=4 f=4 d=4 x=4 p=2
 *
 * Floating point is not implemented.
 */

#define INTTMP 0x02E0
#define INTVAR 0x0E00
#define FLTTMP 0x0001
#define FLTVAR 0x0001
#define INTRET 0x01000  /* R12 */
#define FLTRET 0x01000  /* R12 */
#define NUM_IREGS       15
#define REG_RETVAL      12       /* r12: return value */
#define REG_FIRST_ARG   2       /* r2: first argument register */
#define REG_LAST_ARG    4       /* r4: last argument register */
#define REG_FIRST_TEMP  5       /* r5: first temp register */
#define SZ_REG_FIRST_TEMP "5"   /* r5: first temp register */
#define REG_LAST_TEMP   8       /* r8: last temp register */
#define REG_SP          13      /* r13, sp: stack pointer */
#define REG_RETADDR     14      /* r14: return address */
#define NUM_ARG_REGS    (REG_LAST_ARG - REG_FIRST_ARG + 1)

#define INT_CALLEE_SAVE (INTVAR | ((unsigned)1 << REG_RETADDR))

#define readsreg(p) \
        (generic((p)->op)==INDIR && (p)->kids[0]->op==VREG+P)
#define setsrc(d) ((d) && (d)->x.regnode && \
        (d)->x.regnode->set == src->x.regnode->set && \
        (d)->x.regnode->mask&src->x.regnode->mask)

#define relink(a, b) ((b)->x.prev = (a), (a)->x.next = (b))

  // std lburg code - Do not change that !
#include "c.h"
#include <time.h>

#define NODEPTR_TYPE Node
#define OP_LABEL(p) ((p)->op)
#define LEFT_CHILD(p) ((p)->kids[0])
#define RIGHT_CHILD(p) ((p)->kids[1])
#define STATE_LABEL(p) ((p)->x.state)


static void address(Symbol, Symbol, long); // Same as Mips and XR16

static void blkfetch(int, int, int, int);
static void blkloop(int, int, int, int, int, int[]);
static void blkstore(int, int, int, int);
static void defaddress(Symbol);
static void defconst(int, int, Value);
static void defstring(int, char *);
static void defsymbol(Symbol);
static void doarg(Node);
static void emit2(Node);
static void export(Symbol);
static void clobber(Node);
static void function(Symbol, Symbol [], Symbol [], int);
static void global(Symbol);
static void import(Symbol);
static void local(Symbol);
static void progbeg(int, char **);
static void progend(void);
static void segment(int);
static void space(int);
static void target(Node);
static int no_fp_support();
static int      bitcount       (unsigned);
static Symbol   argreg         (int, int, int, int, int);

static Symbol ireg[32], lreg[32], freg2[32], r2r3, r3r4;
static Symbol iregw, lregw, freg2w;
static int tmpregs[] = {1, REG_FIRST_TEMP+1, REG_FIRST_TEMP+2};
static Symbol blkreg;

static int gnum = 8;

static int cseg;

%}
%start stmt

%term CNSTF4=4113
%term CNSTI1=1045 CNSTI2=2069 CNSTI4=4117
%term CNSTP2=2071
%term CNSTU1=1046 CNSTU2=2070 CNSTU4=4118

%term ARGB=41
%term ARGF4=4129
%term ARGI2=2085 ARGI4=4133
%term ARGP2=2087
%term ARGU2=2086 ARGU4=4134

%term ASGNB=57
%term ASGNF4=4145
%term ASGNI1=1077 ASGNI2=2101 ASGNI4=4149
%term ASGNP2=2103
%term ASGNU1=1078 ASGNU2=2102 ASGNU4=4150

%term INDIRB=73
%term INDIRF4=4161
%term INDIRI1=1093 INDIRI2=2117 INDIRI4=4165
%term INDIRP2=2119
%term INDIRU1=1094 INDIRU2=2118 INDIRU4=4166

%term CVFF4=4209
%term CVFI2=2165 CVFI4=4213

%term CVIF4=4225
%term CVII1=1157 CVII2=2181 CVII4=4229
%term CVIU1=1158 CVIU2=2182 CVIU4=4230

%term CVPU2=2198

%term CVUI1=1205 CVUI2=2229 CVUI4=4277
%term CVUP2=2231
%term CVUU1=1206 CVUU2=2230 CVUU4=4278

%term NEGF4=4289
%term NEGI2=2245 NEGI4=4293

%term CALLB=217
%term CALLF4=4305
%term CALLI2=2261 CALLI4=4309
%term CALLP2=2263
%term CALLU2=2262 CALLU4=4310
%term CALLV=216

%term RETF4=4337
%term RETI2=2293 RETI4=4341
%term RETP2=2295
%term RETU2=2294 RETU4=4342
%term RETV=248

%term ADDRGP2=2311

%term ADDRFP2=2327

%term ADDRLP2=2343

%term ADDF4=4401
%term ADDI2=2357 ADDI4=4405
%term ADDP2=2359
%term ADDU2=2358 ADDU4=4406

%term SUBF4=4417
%term SUBI2=2373 SUBI4=4421
%term SUBP2=2375
%term SUBU2=2374 SUBU4=4422

%term LSHI2=2389 LSHI4=4437
%term LSHU2=2390 LSHU4=4438

%term MODI2=2405 MODI4=4453
%term MODU2=2406 MODU4=4454

%term RSHI2=2421 RSHI4=4469
%term RSHU2=2422 RSHU4=4470

%term BANDI2=2437 BANDI4=4485
%term BANDU2=2438 BANDU4=4486

%term BCOMI2=2453 BCOMI4=4501
%term BCOMU2=2454 BCOMU4=4502

%term BORI2=2469 BORI4=4517
%term BORU2=2470 BORU4=4518

%term BXORI2=2485 BXORI4=4533
%term BXORU2=2486 BXORU4=4534

%term DIVF4=4545
%term DIVI2=2501 DIVI4=4549
%term DIVU2=2502 DIVU4=4550

%term MULF4=4561
%term MULI2=2517 MULI4=4565
%term MULU2=2518 MULU4=4566

%term EQF4=4577
%term EQI2=2533 EQI4=4581
%term EQU2=2534 EQU4=4582

%term GEF4=4593
%term GEI2=2549 GEI4=4597
%term GEU2=2550 GEU4=4598

%term GTF4=4609
%term GTI2=2565 GTI4=4613
%term GTU2=2566 GTU4=4614

%term LEF4=4625
%term LEI2=2581 LEI4=4629
%term LEU2=2582 LEU4=4630

%term LTF4=4641
%term LTI2=2597 LTI4=4645
%term LTU2=2598 LTU4=4646

%term NEF4=4657
%term NEI2=2613 NEI4=4661
%term NEU2=2614 NEU4=4662

%term JUMPV=584

%term LABELV=600

%term LOADB=233
%term LOADF4=4321
%term LOADI1=1253 LOADI2=2277 LOADI4=4325
%term LOADP2=2279
%term LOADU1=1254 LOADU2=2278 LOADU4=4326

%term VREGP=711
%%

reg:  INDIRI1(VREGP)     "# read register\n"
reg:  INDIRU1(VREGP)     "# read register\n"

reg:  INDIRI2(VREGP)     "# read register\n"
reg:  INDIRU2(VREGP)     "# read register\n"

reg:  INDIRF4(VREGP)     "# read register\n" no_fp_support()
reg:  INDIRI4(VREGP)     "# read register\n"
reg:  INDIRP2(VREGP)     "# read register\n"
reg:  INDIRU4(VREGP)     "# read register\n"

stmt: ASGNI1(VREGP,reg)  "# write register\n"
stmt: ASGNU1(VREGP,reg)  "# write register\n"

stmt: ASGNI2(VREGP,reg)  "# write register\n"
stmt: ASGNU2(VREGP,reg)  "# write register\n"
stmt: ASGNP2(VREGP,reg)  "# write register\n"

stmt: ASGNF4(VREGP,reg)  "# write register\n" no_fp_support()
stmt: ASGNI4(VREGP,reg)  "# write register\n"
stmt: ASGNU4(VREGP,reg)  "# write register\n"


stmt: reg  ""
con: CNSTI1  "%a"
con: CNSTU1  "%a"
con: CNSTI2  "%a"
con: CNSTU2  "%a"
con: CNSTP2  "%a"

acon: con     "%0"
acon: ADDRGP2 "%a"  

con: CNSTI1  "%a"
con: CNSTU1  "%a"
con: CNSTI2  "%a"
con: CNSTU2  "%a"
con: CNSTP2  "%a"

reg: CNSTI1  "# reg\n"  range(a, 0, 0) 
reg: CNSTI2  "# reg\n"  range(a, 0, 0)
reg: CNSTI4  "# reg\n"  range(a, 0, 0)
reg: CNSTU1  "# reg\n"  range(a, 0, 0)
reg: CNSTU2  "# reg\n"  range(a, 0, 0)
reg: CNSTU4  "# reg\n"  range(a, 0, 0)
reg: CNSTP2  "# reg\n"  range(a, 0, 0)
k4:  CNSTI1  "%a" range(a,-8,7)
k4:  CNSTI2  "%a" range(a,-8,7)

uk4: CNSTU1  "%a" range(a,0,15)
uk4: CNSTU2  "%a" range(a,0,31)

addr: ADDI2(reg,acon)  "%1(r%0)"
addr: ADDU2(reg,acon)  "%1(r%0)"
addr: ADDP2(reg,acon)  "%1(r%0)"

reg: addr  "zlea r%c,%0\n"  1
addr: acon  "%0"
addr: reg   "(r%0)"
addr: ADDRFP2  "%a+%F(sp)"
addr: ADDRLP2  "%a+%F(sp)"

stmt: ASGNI1(addr,reg)  "sb r%1,%0\n"  1
stmt: ASGNU1(addr,reg)  "sb r%1,%0\n"  1
stmt: ASGNI2(addr,reg)  "sw r%1,%0\n"  1
stmt: ASGNU2(addr,reg)  "sw r%1,%0\n"  1
stmt: ASGNI4(addr,reg)  "zsl r%1,%0\n"  2
stmt: ASGNU4(addr,reg)  "zsl r%1,%0\n"  2
stmt: ASGNP2(addr,reg)  "sw r%1,%0\n"  1
reg:  INDIRI1(addr)     "lbs r%c,%0\n"  1
reg:  INDIRU1(addr)     "lbu r%c,%0\n"  1
reg:  INDIRI2(addr)     "lw r%c,%0\n"  1
reg:  INDIRU2(addr)     "lw r%c,%0\n"  1
reg:  INDIRI4(addr)     "zll r%c,%0\n"  2
reg:  INDIRU4(addr)     "zll r%c,%0\n"  2
reg:  INDIRP2(addr)     "lw r%c,%0\n"  1
reg:  CVII2(INDIRI1(addr))     "lbs r%c,%0\n"  1
reg:  CVUU2(INDIRU1(addr))     "lbu r%c,%0\n"  1
reg:  CVUI2(INDIRU1(addr))     "lbu r%c,%0\n"  1
reg:  CVII4(INDIRI1(addr))     "zlbsl r%c,%0\n"  2
reg:  CVII4(INDIRI2(addr))     "zlwsl r%c,%0\n"  2
reg:  CVUU4(INDIRU1(addr))     "zlbul r%c,%0\n"  2
reg:  CVUU4(INDIRU2(addr))     "zlwul r%c,%0\n"  2
reg:  CVUI4(INDIRU1(addr))     "zlbul r%c,%0\n"  2
reg:  CVUI4(INDIRU2(addr))     "zlwul r%c,%0\n"  2
reg:  INDIRF4(addr)     "# fp\n"  no_fp_support()
stmt: ASGNF4(addr,reg)  "# fp\n"  no_fp_support()
reg: ADDI2(reg,reg)   "add r%c,r%0,r%1\n"  1
reg: ADDU2(reg,reg)   "add r%c,r%0,r%1\n"  1
reg: ADDP2(reg,reg)   "add r%c,r%0,r%1\n"  1
reg: ADDI4(reg,reg)   "zaddl r%c,r%0,r%1\n"  2
reg: ADDU4(reg,reg)   "zaddl r%c,r%0,r%1\n"  2

reg: ADDI2(reg,k4)   "addi r%c,r%0,%1\n"  1
reg: ADDI4(reg,k4)   "addil r%c,r%0,%1\n"  1
reg: ADDP2(reg,k4)   "addi r%c,r%0,%1\n"  1
reg: ADDU2(reg,k4)   "addi r%c,r%0,%1\n"  1
reg: ADDU4(reg,k4)   "addil r%c,r%0,%1\n"  1
reg: SUBI2(reg,reg)   "sub r%c,r%0,r%1\n"  1
reg: SUBP2(reg,reg)   "sub r%c,r%0,r%1\n"  1
reg: SUBU2(reg,reg)   "sub r%c,r%0,r%1\n"  1
reg: SUBI4(reg,reg)   "zsubl r%c,r%0,r%1\n"  2
reg: SUBU4(reg,reg)   "zsubl r%c,r%0,r%1\n"  2
reg: SUBI2(reg,k4)   "subi r%c,r%0,%1\n"  1
reg: SUBU2(reg,k4)   "subi r%c,r%0,%1\n"  1
reg: SUBP2(reg,k4)   "subi r%c,r%0,%1\n"  1
reg: SUBI4(reg,k4)   "zsubil r%c,r%0,%1\n"  2
reg: SUBU4(reg,k4)   "zsubil r%c,r%0,%1\n"  2
reg: NEGI2(reg)   "sub r%c,r0,r%0\n"  1
reg: NEGI4(reg)   "subl r%c,r0,r%0\n"  1
reg: MULI2(reg,reg)  "zcall _muli2\n"   1
reg: MULU2(reg,reg)  "zcall _mulu2\n"   1
reg: MULI4(reg,reg)  "zcall _muli4\n"   1
reg: MULU4(reg,reg)  "zcall _mulu4\n"   1
reg: DIVI2(reg,reg)  "zcall _divi2\n"   1
reg: DIVU2(reg,reg)  "zcall _divu2\n"  1
reg: DIVI4(reg,reg)  "zcall _divi4\n"   1
reg: DIVU4(reg,reg)  "zcall _divu4\n"  1
reg: MODI2(reg,reg)  "zcall _modi2\n"   1
reg: MODU2(reg,reg)  "zcall _modu2\n"  1
reg: MODI4(reg,reg)  "zcall _modi4\n"   1
reg: MODU4(reg,reg)  "zcall _modu4\n"  1
reg: RSHI2(reg,reg)  "?zmov r%c,r%0\nsra r%c,r%1\n"  1
reg: RSHU2(reg,reg)  "?zmov r%c,r%0\nsrl r%c,r%1\n"  1
reg: RSHI2(reg,uk4)  "?zmov r%c,r%0\nsrai r%c,%1\n"  1
reg: RSHU2(reg,uk4)  "?zmov r%c,r%0\nsrli r%c,%1\n"  1
reg: RSHI4(reg,reg)  "call _sral\n"  10
reg: RSHU4(reg,reg)  "call _srll\n"  10
reg: LSHI2(reg,reg)  "?zmov r%c,r%0\nsla r%c,r%1\n"  1
reg: LSHU2(reg,reg)  "?zmov r%c,r%0\nsll r%c,r%1\n"  1

reg: LSHI2(reg,uk4)  "?zmov r%c,r%0\nslai r%c,r%1\n"  1
reg: LSHU2(reg,uk4)  "?zmov r%c,r%0\nslli r%c,r%1\n"  1
reg: BANDI2(reg,reg)  "?zmov r%c,r%0\nand r%c,r%1\n"   1
reg: BANDI4(reg,reg)  "?zmovl r%c,r%0\nzandl r%c,r%1\n"   2
reg: BANDU2(reg,reg)  "?zmov r%c,r%0\nand r%c,r%1\n"   1
reg: BANDU4(reg,reg)  "?zmovl r%c,r%0\nzandl r%c,r%1\n"   2
reg: BORI2(reg,reg)  "?zmov r%c,r%0\nor r%c,r%1\n"   1
reg: BORI4(reg,reg)  "?zmovl r%c,r%0\nzorl r%c,r%1\n"   2
reg: BORU2(reg,reg)  "?zmov r%c,r%0\nor r%c,r%1\n"   1
reg: BORU4(reg,reg)  "?zmovl r%c,r%0\nzorl r%c,r%1\n"   2
reg: BXORI2(reg,reg)  "?zmov r%c,r%0\nxor r%c,r%1\n"   1
reg: BXORI4(reg,reg)  "?zmovl r%c,r%0\nzxorl r%c,r%1\n"   2
reg: BXORU2(reg,reg)  "?zmov r%c,r%0\nxor r%c,r%1\n"   1
reg: BXORU4(reg,reg)  "?zmovl r%c,r%0\nzxorl r%c,r%1\n"   2
reg: BCOMI2(reg)  "?zmov r%c,r%0\nxor r%c,r0\n"   1
reg: BCOMI4(reg)  "?zmovl r%c,r%0\nzxorl r%c,r0\n"   2
reg: BCOMU2(reg)  "?zmov r%c,r%0\nxor r%c,r0\n"   1
reg: BCOMU4(reg)  "?zmovl r%c,r%0\nzxorl r%c,r0\n"   2
reg: LOADI1(reg)  "zmov r%c,r%0\n"  move(a)
reg: LOADU1(reg)  "zmov r%c,r%0\n"  move(a)
reg: LOADI2(reg)  "zmov r%c,r%0\n"  move(a)
reg: LOADU2(reg)  "zmov r%c,r%0\n"  move(a)
reg: LOADI4(reg)  "zmov r%c,r%0\n"  move(a)
reg: LOADP2(reg)  "zmov r%c,r%0\n"  move(a)
reg: LOADU4(reg)  "zmov r%c,r%0\n"  move(a)
reg: CVFF4(reg)  ""  no_fp_support()
reg: CVIF4(reg)  ""  no_fp_support()
reg: CVFI2(reg)  ""  no_fp_support()
reg: CVFI4(reg)  ""  no_fp_support()
reg: ADDF4(reg,reg)  ""  no_fp_support()
reg: DIVF4(reg,reg)  ""  no_fp_support()
reg: MULF4(reg,reg)  ""  no_fp_support()
reg: SUBF4(reg,reg)  ""  no_fp_support()
reg: LOADF4(reg)     ""  move(a)+no_fp_support()
reg: NEGF4(reg)      ""  no_fp_support()
stmt: LABELV "%a:\n"
stmt: JUMPV(acon)  "br %0 - yeah boss\n"   1
stmt: JUMPV(reg)   "jal r0,0(r%0)\n"  1
stmt: EQI2(reg,reg)  "zbeq r%0,r%1,%a\n"   2
stmt: EQU2(reg,reg)  "zbeq r%0,r%1,%a\n"   2
stmt: EQI4(reg,reg)  "zbeql r%0,r%1,%a\n"  2
stmt: EQU4(reg,reg)  "zbeql r%0,r%1,%a\n"  2
stmt: NEI2(reg,reg)  "zbne  r%0,r%1,%a\n"   2
stmt: NEU2(reg,reg)  "zbne  r%0,r%1,%a\n"   2
stmt: NEI4(reg,reg)  "zbnel  r%0,r%1,%a\n"   2
stmt: NEU4(reg,reg)  "zbnel  r%0,r%1,%a\n"   2
stmt: GEI2(reg,reg)  "zbge  r%0,r%1,%a\n"  2
stmt: GEU2(reg,reg)  "zbgeu r%0,r%1,%a\n"  2
stmt: GEI4(reg,reg)  "zbgel  r%0,r%1,%a\n"  2
stmt: GEU4(reg,reg)  "zbgeul r%0,r%1,%a\n"  2
stmt: GTI2(reg,reg)  "zbgt  r%0,r%1,%a\n"  2
stmt: GTU2(reg,reg)  "zbgtu r%0,r%1,%a\n"  2
stmt: GTU4(reg,reg)  "zbgtul r%0,r%1,%a\n"  2
stmt: GTI4(reg,reg)  "zbgtl  r%0,r%1,%a\n"  2
stmt: LEI2(reg,reg)  "zble  r%0,r%1,%a\n"  2
stmt: LEU2(reg,reg)  "zbleu r%0,r%1,%a\n"  2
stmt: LEI4(reg,reg)  "zblel  r%0,r%1,%a\n"  2
stmt: LEU4(reg,reg)  "zbleul r%0,r%1,%a\n"  2
stmt: LTI2(reg,reg)  "zblt  r%0,r%1,%a\n"   2
stmt: LTU2(reg,reg)  "zbltu r%0,r%1,%a\n"   2
stmt: LTI4(reg,reg)  "zbltl  r%0,r%1,%a\n"   2
stmt: LTU4(reg,reg)  "zbltul r%0,r%1,%a\n"   2
stmt: EQF4(reg,reg)  ""  no_fp_support()
stmt: LEF4(reg,reg)  ""  no_fp_support()
stmt: LTF4(reg,reg)  ""  no_fp_support()
stmt: GEF4(reg,reg)  ""  no_fp_support()
stmt: GTF4(reg,reg)  ""  no_fp_support()
stmt: NEF4(reg,reg)  ""  no_fp_support()
call_addr:   ADDRGP2 "%a"
call_addr:   CNSTP2  "%a"   range(a, 0, 0xffff)
reg:  CALLF4(call_addr)  "zcall %0\n"  no_fp_support()
reg:  CALLI2(call_addr)  "zcall %0\n"  2
reg:  CALLI4(call_addr)  "zcall %0\n"  2
reg:  CALLP2(call_addr)  "zcall %0\n"  2
reg:  CALLU2(call_addr)  "zcall %0\n"  2
reg:  CALLU4(call_addr)  "zcall %0\n"  2
stmt: CALLV(call_addr)   "zcall %0\n"  2
call_reg:  reg     "r%0"
reg:  CALLF4(call_reg)  "jl %0\n"  no_fp_support()
reg:  CALLI2(call_reg)  "jl %0\n"  1
reg:  CALLI4(call_reg)  "jl %0\n"  1
reg:  CALLP2(call_reg)  "jl %0\n"  1
reg:  CALLU2(call_reg)  "jl %0\n"  1
reg:  CALLU4(call_reg)  "jl %0\n"  1
stmt: CALLV(call_reg)   "jl %0\n"  1
stmt: RETI2(reg)  "# ret\n"  1
stmt: RETI4(reg)  "# ret\n"  1
stmt: RETU2(reg)  "# ret\n"  1
stmt: RETU4(reg)  "# ret\n"  1
stmt: RETP2(reg)  "# ret\n"  1
stmt: RETV(reg)   "# ret\n"  1
stmt: RETF4(reg)  "# ret\n"  no_fp_support()
stmt: ARGF4(reg)  "# arg\n"  fp()
stmt: ARGI2(reg)  "# arg\n"  1
stmt: ARGI4(reg)  "# arg\n"  1
stmt: ARGP2(reg)  "# arg\n"  1
stmt: ARGU2(reg)  "# arg\n"  1
stmt: ARGU4(reg)  "# arg\n"  1
stmt: ARGB(INDIRB(reg))       "# zargb %0 - not supported\n"      1
stmt: ASGNB(reg,INDIRB(reg))  "# zasgnb %0 %1\n"  1
%%

static void progbeg(int argc, char *argv[]) {
        int i;
        time_t now;
        struct tm* ptmNow;
        static char rev[] = "0.1";
 
        {
                union {
                        char c;
                        int i;
                } u;
                u.i = 0;
                u.c = 1;
                swap = ((int)(u.i == 1)) != IR->little_endian;
        }

        time(&now);
        ptmNow = localtime(&now);
        rev[strlen(rev)-2] = 0;
        printf("; generated by lcc-ctm16 rev %s on %s\n", rev, asctime(ptmNow));

        parseflags(argc, argv);
        for (i = 0; i < argc; i++)
                if (strncmp(argv[i], "-G", 2) == 0)
                        gnum = atoi(argv[i] + 2);
        for (i = 0; i < NUM_IREGS; i++)
                ireg[i] = mkreg("%d", i, 1, IREG);
        for (i = 0; i < NUM_IREGS; i += 2)
                lreg[i] = mkreg("%d", i, 3, IREG);
        for (i = 0; i < 31; i += 2)
                freg2[i] = mkreg("%d", i, 3, FREG);
        ireg[REG_SP]->x.name = "sp";
        r2r3 = mkreg("2", 2, 3, IREG);
        r3r4 = mkreg("3", 3, 3, IREG);
        freg2w = mkwildcard(freg2); /* Unused */
        iregw = mkwildcard(ireg);   /* wildcard for 16-bit (native) registers */
        lregw = mkwildcard(lreg);   /* wildcard for register pairs */
  
      tmask[IREG] = INTTMP; tmask[FREG] = FLTTMP;
      vmask[IREG] = INTVAR; vmask[FREG] = FLTVAR;
        blkreg = mkreg(SZ_REG_FIRST_TEMP, REG_FIRST_TEMP, 7, IREG);
}

static void progend(void){}
static Symbol rmap(int opk) {
        switch (optype(opk)) {
        case I: case U:
                return (opsize(opk) == 4) ? lregw : iregw;
        case P: case B:
                return iregw;
        case F:
                no_fp_support();
                return freg2w;
        default:
                return 0;
        }
}

static void target(Node p) {
        assert(p);
        switch (specific(p->op)) {
        case CNST+I: case CNST+U: case CNST+P:
                if (range(p, 0, 0) == 0) {
                        setreg(p, ireg[0]);
                        p->x.registered = 1;
                }
                break;
 
        case CALL+I: case CALL+P: case CALL+U:
                setreg(p, ireg[REG_RETVAL]);
                break;
        case CALL+V:
                break;
        case CALL+F:                                                                                                              no_fp_support();                    
        case RET+I: case RET+U: case RET+P:
                rtarget(p, 0, ireg[REG_RETVAL]); /* Force  r12 */
                break;
        case RET+F:
                no_fp_support();
                break;
case ARG+F: case ARG+I: case ARG+P: case ARG+U: {
                static int ty0;
                int ty = optype(p->op);
                Symbol q;

                q = argreg(p->x.argno, p->syms[2]->u.c.v.i, ty, opsize(p->op), ty0);
                if (p->x.argno == 0)
                        ty0 = ty;
                if (q &&
                !(ty == F && q->x.regnode->set == IREG))
                        rtarget(p, 0, q);
                break;
                }
case ARG+B:  rtarget(p->kids[0], 0, blkreg); break;
        case ASGN+B: rtarget(p->kids[1], 0, blkreg); break;
        case DIV+I: case MOD+I: case MUL+I:
        case DIV+U: case MOD+U: case MUL+U:
                setreg(p, ireg[REG_RETVAL]);
                rtarget(p, 0, ireg[REG_FIRST_ARG]);
                rtarget(p, 1, ireg[REG_FIRST_ARG+1]);
                break;
        }
}


static void clobber(Node p) {
        assert(p);
        switch (specific(p->op)) {
        case CALL+F:
                spill(INTTMP | INTRET, IREG, p);
                spill(FLTTMP,          FREG, p);
                break;
        case CALL+I: case CALL+P: case CALL+U:
                spill(INTTMP,          IREG, p);
                spill(FLTTMP | FLTRET, FREG, p);
                break;
        case CALL+V:
                spill(INTTMP | INTRET, IREG, p);
                spill(FLTTMP | FLTRET, FREG, p);
                break;
        }
}
static void emit2(Node p) {
        int dst, n, src, sz, ty;
        static int ty0;
        Symbol q;

        switch (specific(p->op)) {
        case ARG+F: case ARG+I: case ARG+P: case ARG+U:
                ty = optype(p->op);
                sz = opsize(p->op);
                if (p->x.argno == 0)
                        ty0 = ty;
                q = argreg(p->x.argno, p->syms[2]->u.c.v.i, ty, sz, ty0);
                src = getregnum(p->x.kids[0]);
                if (q == NULL && ty == F && sz == 4)
                        print("s.s rf%d,%d(sp)\n", src, p->syms[2]->u.c.v.i);
                else if (q == NULL && ty == F)
                        print("s.d rf%d,%d(sp)\n", src, p->syms[2]->u.c.v.i);
                else if (q == NULL)
                        print("s%c r%d,%d(sp)\n", "wl"[sz==4], src, p->syms[2]->u.c.v.i);
                else if (ty == F && sz == 4 && q->x.regnode->set == IREG)
                        print("mfc1 r%d,rf%d\n", q->x.regnode->number, src);
                else if (ty == F && q->x.regnode->set == IREG)
                        print("mfc1.d r%d,rf%d\n", q->x.regnode->number, src);
                break;
        case ASGN+B:
                dalign = salign = p->syms[1]->u.c.v.i;
                blkcopy(getregnum(p->x.kids[0]), 0,
                        getregnum(p->x.kids[1]), 0,
                        p->syms[0]->u.c.v.i, tmpregs);
                break;
        case ARG+B:
                dalign = 4;
                salign = p->syms[1]->u.c.v.i;
                blkcopy(REG_SP, p->syms[2]->u.c.v.i,
                        getregnum(p->x.kids[0]), 0,
                        p->syms[0]->u.c.v.i, tmpregs);
                n   = p->syms[2]->u.c.v.i + p->syms[0]->u.c.v.i;
                dst = p->syms[2]->u.c.v.i;
                for ( ; dst <= (4*(NUM_ARG_REGS-1)) && dst < n; dst += 4)
                        print("lw r%d,%d(sp)\n", (dst/4)+4, dst);
                break;
        }
}
static Symbol argreg(int argno, int offset, int ty, int sz, int ty0) {
        assert((offset&1) == 0);
        if (offset + sz > 2*NUM_ARG_REGS || !(ty == I || ty == U || ty == P))
                return NULL;
        else if (argno == 0 && sz == 4)
                return r2r3;
        else if (argno == 1 && sz == 4)
                return r3r4;
        else
                return ireg[(offset/2) + REG_FIRST_ARG];
}
static void doarg(Node p) {
        static int argno;
        int align;

        if (argoffset == 0)
                argno = 0;
        p->x.argno = argno++;
        align = p->syms[1]->u.c.v.i < 2 ? 2 : p->syms[1]->u.c.v.i;
        p->syms[2] = intconst(mkactual(align,
                p->syms[0]->u.c.v.i));
}
static void local(Symbol p) {
        if (askregvar(p, rmap(ttob(p->type))) == 0)
                mkauto(p);
}
static void function(Symbol f, Symbol caller[], Symbol callee[], int ncalls) {
        int i, saved, sizefsave, sizeisave, varargs;
        Symbol r, argregs[NUM_ARG_REGS];

        usedmask[0] = usedmask[1] = 0;
        freemask[0] = freemask[1] = ~(unsigned)0;
        offset = maxoffset = maxargoffset = 0;
        for (i = 0; callee[i]; i++)
                ;
        varargs = variadic(f->type)
                || i > 0 && strcmp(callee[i-1]->name, "va_alist") == 0;
        for (i = 0; callee[i]; i++) {
                Symbol p = callee[i];
                Symbol q = caller[i];
                assert(q);
                offset = roundup(offset, q->type->align);
                p->x.offset = q->x.offset = offset;
                p->x.name = q->x.name = stringd(offset);
                r = argreg(i, offset, optype(ttob(q->type)), q->type->size, optype(ttob(caller[0]->type)));
                if (i < NUM_ARG_REGS)
                        argregs[i] = r;
                offset = roundup(offset + q->type->size, 2);
                if (varargs)
                        p->sclass = AUTO;
                else if (r && ncalls == 0 &&
                         !isstruct(q->type) && !p->addressed &&
                         !(isfloat(q->type) && r->x.regnode->set == IREG)
) {
                        p->sclass = q->sclass = REGISTER;
                        askregvar(p, r);
                        assert(p->x.regnode && p->x.regnode->vbl == p);
                        q->x = p->x;
                        q->type = p->type;
                }
                else if (askregvar(p, rmap(ttob(p->type)))
                         && r != NULL
                         && (isint(p->type) || p->type == q->type)) {
                        assert(q->sclass != REGISTER);
                        p->sclass = q->sclass = REGISTER;
                        q->type = p->type;
                }
        }
        assert(!caller[i]);
        offset = 0;
        gencode(caller, callee);
        if (ncalls)
                usedmask[IREG] |= ((unsigned)1)<<REG_RETADDR;
        usedmask[IREG] &= INT_CALLEE_SAVE;
        usedmask[FREG] &= 0xfff00000;
        maxargoffset = roundup(maxargoffset, usedmask[FREG] ? 8 : 2);
        if (ncalls && maxargoffset < NUM_ARG_REGS*2)
                maxargoffset = NUM_ARG_REGS*2;
        sizefsave = 4*bitcount(usedmask[FREG]);
        sizeisave = 2*bitcount(usedmask[IREG]);
        framesize = roundup(maxargoffset + sizefsave
                + sizeisave + maxoffset, 2);
        segment(CODE);
        printf("align 16\n");
        printf("%s:\n", f->x.name);
        i = maxargoffset + sizefsave - framesize;
        if (framesize > 0)
                print("addi sp,sp,%d\n", -framesize);
        saved = maxargoffset;
        for (i = 20; i <= 30; i += 2)
                if (usedmask[FREG]&(3<<i)) {
                        print("s.d rf%d,%d(sp)\n", i, saved);
                        saved += 8;
                }

        for (i = 0; i < NUM_IREGS; i++)
                if (usedmask[IREG]&(1<<i)) {
                        print("sw r%d,%d(sp)\n", i, saved);
                        saved += 2;
                }
        for (i = 0; i < NUM_ARG_REGS && callee[i]; i++) {
                r = argregs[i];
                if (r && r->x.regnode != callee[i]->x.regnode) {
                        Symbol out = callee[i];
                        Symbol in  = caller[i];
                        int rn = r->x.regnode->number;
                        int rs = r->x.regnode->set;
                        int tyin = ttob(in->type);

                        assert(out && in && r && r->x.regnode);
                        assert(out->sclass != REGISTER || out->x.regnode);
                        if (out->sclass == REGISTER
                        && (isint(out->type) || out->type == in->type)) {
                                int outn = out->x.regnode->number;
                                if (rs == FREG && tyin == F+sizeop(8))
                                        print("mov.d rf%d,rf%d\n", outn, rn);
                                else if (rs == FREG && tyin == F+sizeop(4))
                                        print("mov.s rf%d,rf%d\n", outn, rn);
                                else if (rs == IREG && tyin == F+sizeop(8))
                                        print("mtc1.d r%d,rf%d\n", rn,   outn);
                                else if (rs == IREG && tyin == F+sizeop(4))
                                        print("mtc1 r%d,rf%d\n",   rn,   outn);
                                else
                                        print("mov r%d,r%d\n",    outn, rn);
                        } else {
                                int off = in->x.offset + framesize;
                                if (rs == FREG && tyin == F+sizeop(8))
                                        print("s.d rf%d,%d(sp)\n", rn, off);
                                else if (rs == FREG && tyin == F+sizeop(4))
                                        print("s.s rf%d,%d(sp)\n", rn, off);
                                else {
                                        int i, n = (in->type->size + 1)/2;
                                        for (i = rn; i < rn+n && i <= REG_LAST_ARG; i++)
                                                print("sw r%d,%d(sp)\n", i, off + (i-rn)*2);
                                }
                        }
                }
        }
        if (varargs && callee[i-1]) {
                i = callee[i-1]->x.offset + callee[i-1]->type->size;
                for (i = roundup(i, 2)/2; i < NUM_ARG_REGS; i++)
                        print("sw r%d,%d(sp)\n", REG_FIRST_ARG + i, framesize + 2*i);
                }
        emitcode();
        saved = maxargoffset;
        for (i = 20; i <= 30; i += 2)
                if (usedmask[FREG]&(3<<i)) {
                        print("l.d rf%d,%d(sp)\n", i, saved);
                        saved += 8;
                }
        for (i = 0; i < NUM_IREGS; i++)
                if (usedmask[IREG]&(1<<i)) {
                        print("lw r%d,%d(sp)\n", i, saved);
                        saved += 2;
                }
        if (framesize > 0)
                print("addi sp,sp,%d\n", framesize);
        print("ret\n\n");
}
static void defconst(int suffix, int size, Value v) {
        if (suffix == F)
                no_fp_support();
        else if (suffix == P)
                print("word %u\n", v.p);
        else if (size == 1)
                print("byte %u\n", suffix == I ? v.i : v.u);
        else if (size == 2)
                print("word %u\n", suffix == I ? v.i : v.u);
        else if (size == 4)
                print("dword %u\n", suffix == I ? v.i : v.u);
}
static void defaddress(Symbol p) {
  print(";;-I- define a address in assembly code\n");
  print(";; Fixme : what is the syntax here ?\n");
  print(".word %s\n", p->x.name);
}
static void defstring(int n, char *str) {
        char *s;
        print(";;-I- define a string  in assembly code\n");
        print(";; Fixme : what is the syntax here ?\n");
        for (s = str; s < str + n; s++)
                print("byte %d\n", (*s)&0377);
}
static void export(Symbol p) {
        print("global %s\n", p->x.name);
}
static void import(Symbol p) {
        if (!isfunc(p->type))
                print("global %s\n", p->x.name); /* good enough? */
}
static void defsymbol(Symbol p) {
        if (p->scope >= LOCAL && p->sclass == STATIC)
                p->x.name = stringf("L%d", genlabel(1));
        else if (p->generated)
                p->x.name = stringf("L%s", p->name);
        else if (p->scope == GLOBAL || p->sclass == EXTERN)
                p->x.name = stringf("_%s", p->name);
        else
                assert(p->scope != CONSTANTS || isint(p->type) || isptr(p->type)),
                p->x.name = p->name;
}

// Same as the Misp & xr16 port
static void address(Symbol q, Symbol p, long n) {
        if (p->scope == GLOBAL
        || p->sclass == STATIC || p->sclass == EXTERN)
                q->x.name = stringf("%s%s%D", p->x.name,
                        n >= 0 ? "+" : "", n);
        else {
                assert(n <= INT_MAX && n >= INT_MIN);
                q->x.offset = p->x.offset + n;
                q->x.name = stringd(q->x.offset);
        }
}
static void global(Symbol p) {
        if (p->type->align > 1)
                print("align %d\n", p->type->align);
        print("%s:\n", p->x.name);
        if (p->u.seg == BSS)
                printf("bss %d\n", p->type->size);
}
static void segment(int n) {
        cseg = n;
}
static void space(int n) {
        if (cseg != BSS)
                print("space %d\n", n);
}

// 
static void blkloop(int dreg, int doff, int sreg, int soff, int size, int tmps[]) {
        int lab = genlabel(1);

        print(";; blkloop called : dreg=%d, doff=%d, sreg=%d, soff=%d, size=%d\n", dreg,doff,sreg,soff,size);
        print(";;; Not implemented yet");
        print("addi r%d,r%d,%d\n", sreg, sreg, size&~7);
        print("addi r%d,r%d,%d\n", tmps[2], dreg, size&~7);
        blkcopy(tmps[2], doff, sreg, soff, size&7, tmps);
        print("L%d:\n", lab);
        print("addi r%d,r%d,%d\n", sreg, sreg, -8);
        print("addi r%d,r%d,%d\n", tmps[2], tmps[2], -8);
        blkcopy(tmps[2], doff, sreg, soff, 8, tmps);
        print("cmp r%d,r%d\nbltu L%d\n", dreg, tmps[2], lab);
}
static void blkfetch(int size, int off, int reg, int tmp) {
        assert(size == 1 || size == 2);
        if (size == 1)
                print("lb r%d,%d(r%d)\n",  tmp, off, reg);
        else {
                assert(salign >= size);
                print("lw r%d,%d(r%d)\n",   tmp, off, reg);
        }
}
static void blkstore(int size, int off, int reg, int tmp) {
        assert(size == 1 || size == 2);
        if (size == 1)
                print("sb r%d,%d(r%d)\n",  tmp, off, reg);
        else {
                assert(dalign >= size);
                print("sw r%d,%d(r%d)\n",  tmp, off, reg);
        }
}
static void stabinit(char *, int, char *[]);
static void stabline(Coordinate *);
static void stabsym(Symbol);

static char *currentfile;

static int bitcount(unsigned mask) {
        unsigned i, n = 0;

        for (i = 1; i; i <<= 1)
                if (mask&i)
                        n++;
        return n;
}

/* stabinit - initialize stab output */
static void stabinit(char *file, int argc, char *argv[]) {
        if (file) {
                print("file \"%s\"\n", file);
                currentfile = file;
        }
}

/* stabline - emit stab entry for source coordinate *cp */
static void stabline(Coordinate *cp) {
        if (cp->file && cp->file != currentfile) {
                print("file \"%s\"\n", cp->file);
                currentfile = cp->file;
        }
        print("loc %d\n", cp->y);
}

/* stabsym - output a stab entry for symbol p */
static void stabsym(Symbol p) {
        if (p == cfunc && IR->stabline)
                (*IR->stabline)(&p->src);
}

// We break if  floatting point is used....
// Fixme : is this tne correct way to handle error ?
static int no_fp_support() {
    static int err = 0;
    if (!err) {
        err = 1;
        error("floating point not implemented\n");
    }
    return 2;
}

Interface ctm16IR = {
  // size, align , outofline
  1, 1, 0,  // char : 1 byte
  2, 2, 0,  // short : 2 bytes
  2, 2, 0,  // int   : 2 bytes 
  4, 4, 0,  // long  : 4 bytes
  4, 4, 0,  // long long : 4 bytes
  4, 4, 1,  // float  : not supported
  4, 4, 1,  // double : not supported
  4, 4, 1,  // long double : not supported
  2, 2, 0,  // T * */
  0, 2, 0,  /* struct */
  0,  // little_endian 
  1,  /* mulops_calls : FIXME : should be 0 - we have multiplication in hardware */
  0,  /* wants_callb  : we don't emit CALLB node for function returning structure */
  0,  /* wants_argb   : we don't emit ARGB node to pass structure as argument  */
  1,  /* left_to_right : present the arguments from left to rigth*/
  0,  /* wants_dag : so we get only trees passed to the backend - std lcc backend behaviour*/
  1,  /* unsigned_char : char ar unsigned by default - TBV */
  address, // Same as the Misp & xr16 port
  blockbeg, // Default LCC function
  blockend, // Default LCC function
  defaddress, // define address symbol in assembly code
  defconst,   // define constant  in assembly code
  defstring,  // define string  in assembly code
  defsymbol,  // define symbol (function, label...) in assembly code
  emit,       // Default LCC function (?)
  export,     // define a symbol to be exported in the assembly file
  function,   // implementation of function - mips and xr16 are very closed - only little differences 
  gen,        // gen (default lcc) ?
  global,     // define a global symbol - with segment handling (difference between MIPS ans xr16)
  import,     // define an extern reference in assembly code
  local,      // define a local variable
  progbeg,    // Set-up gode generator
  progend,    // empty
  segment,    // print segment information (MIPS code more complete than xr16)
  space,      // print "space" section/segment (?)
  0, 0, 0, stabinit, stabline, stabsym, 0, // initialize stab output, emit stab entry for source coordinate, output a stab entry for symbol 
  {
    1,      /* max_unaligned_load  : we can do only "unaligned" access on byte, not 16-bit word*/
    rmap,
    blkfetch, blkstore, blkloop, // function for block copy
    _label,
    _rule,
    _nts,
    _kids,
    _string,
    _templates,
    _isinstruction,
    _ntname,
    emit2,
    doarg,   // same as for mips and xr16
    target,
    clobber,  
  }
};

